#ifndef _OS_AZ_MEMORY_H_
#define _OS_AZ_MEMORY_H_ 1

#include <stdint.h>
#include <stdlib.h>
#include <errno.h>
#include <aznix/az_types.h>

#ifdef __cplusplus
extern "C" {
#endif

/*
 * =============================
 * New aznix Memory system calls
 * =============================
 */

#define AZMM_NR_MEMORY_ACCOUNTS   8
#define AZMM_NR_MEMORY_FUNDS      8

typedef struct {
	int64_t         	ac_balance;
	int64_t        		ac_balance_min;
	int64_t         	ac_balance_max;
	size_t          	ac_allocated;
	size_t          	ac_allocated_min;
	size_t          	ac_allocated_max;
	size_t          	ac_maximum;
	int64_t			ac_accts_to_credit;
	int64_t         	ac_fund;
	int64_t         	ac_overdraft_fund;
} account_stats_t;

typedef struct {
	uint64_t        	ac_count;
    	uint64_t        	proc_mrlimit;
    	account_stats_t 	ac_array[AZMM_NR_MEMORY_ACCOUNTS];
} account_info_t;

typedef struct {
	int64_t         	ac_balance;
	int64_t        		ac_balance_min;
	int64_t         	ac_balance_max;
	size_t          	ac_allocated;
	size_t          	ac_allocated_min;
	size_t          	ac_allocated_max;
	size_t          	ac_maximum;
	int64_t			ac_accts_to_credit;
	int64_t         	ac_fund;
	int64_t         	ac_overdraft_fund;
	int64_t 		ac_subpages_free;
} account_usage_t;

typedef struct {
	uint64_t        	ac_count;
	uint64_t        	proc_mrlimit;
	account_usage_t 	ac_array[AZMM_NR_MEMORY_ACCOUNTS];
} account_usage_info_t;


typedef struct {
    size_t fnd_balance;
    size_t fnd_maximum;
    size_t fnd_limit;
} fund_stats_t;

typedef struct {
    uint64_t     fi_count;
    fund_stats_t fi_array[AZMM_NR_MEMORY_FUNDS];
} funds_info_t;

typedef struct {
	size_t fnd_balance;
	size_t fnd_maximum;
} old_fund_stats_t;

typedef struct {
	uint64_t     fi_count;
	old_fund_stats_t fi_array[AZMM_NR_MEMORY_FUNDS];
} old_funds_info_t;

typedef struct {
	unsigned long		prezeroed_bytes;
	unsigned long		prezeroable_bytes;
	uint64_t                dummy1;  
	uint64_t                dummy2;  
	uint64_t                dummy3; 
	uint64_t                dummy4;
	uint64_t                dummy5;
	uint64_t                dummy6;
	uint64_t                dummy7;
	uint64_t                dummy8;
	uint64_t                dummy9;
	uint64_t                dummy10;
	uint64_t                dummy11;
	uint64_t                dummy12;
} prezero_stats_t;



#define AZ_NR_DEVICES	       16

#define AZMM_SMALL_PAGE_SHIFT  12
#define AZMM_LARGE_PAGE_SHIFT  21

#define AZMM_PUD_SHIFT         30
#define AZMM_PUD_SIZE          (1UL << AZMM_PUD_SHIFT)
#define AZMM_PUD_MASK          (~(AZMM_PUD_SIZE - 1))

#define AZMM_SMALL_PAGE_SIZE   (1UL<<AZMM_SMALL_PAGE_SHIFT) //4 KB
#define AZMM_LARGE_PAGE_SIZE   (1UL<<AZMM_LARGE_PAGE_SHIFT)     //2 MB

#define AZMM_SMALL_PAGE_OFFSET (AZMM_SMALL_PAGE_SIZE - 1)
#define AZMM_SMALL_PAGE_MASK   (~AZMM_SMALL_PAGE_OFFSET)

#define AZ_PAGE_MASK	       (~AZMM_SMALL_PAGE_OFFSET)

#define AZMM_LARGE_PAGE_OFFSET (AZMM_LARGE_PAGE_SIZE - 1)
#define AZMM_LARGE_PAGE_MASK   (~AZMM_LARGE_PAGE_OFFSET)

#define AZMM_NO_TLB_INVALIDATE           	0x0100000
#define AZMM_BLIND_UNMAP                 	0x0200000
#define AZMM_BLIND_PROTECT               	0x0200000
#define AZMM_BLIND_REMAP	               	0x0200000
#define AZMM_LARGE_PAGE_MAPPINGS         	0x0400000
#define AZMM_MAY_SHATTER_LARGE_MAPPINGS  	0x0800000
#define AZMM_BATCHABLE                   	0x1000000
#define AZMM_ALIASABLE                   	0x2000000
#define AZMM_NODUMP				0x04000000
#define AZMM_REPORT_PROT_AS_RWX			0x80000000
#define AZMM_RETAIN_SRC_MAPPINGS	 	0x04000000
#define AZMM_MPROBE_SHADOW               	0x00000010
#define AZMM_MPROBE_RETURN_EMPTY_L1_ADDR	0x00000020
#define AZMM_MPROBE_REF_COUNT		 	0x08000000
#define AZMM_MPROBE_P2P_PFN              	0x20000000
#define AZMM_REFCOUNT_WATCHPOINT_IFNOT512	0x40000000
#define AZMM_DISABLE_REFCOUNT_WATCHPOINT        0x80000000
#define AZMM_PREFER_NODE_LOCAL		 	0x10000000
#define AZMM_DISABLE_SCRUB_AT_UNMAP             0x00080000
#define AZMM_CAN_AVOID_SRUBBING_WITHIN_UID      0x00100000

#define AZMM_MRESERVE_WITH_MMAP_SEMANTICS       0x00100000
#define AZMM_MUNRESERVE_WITH_MUNMAP_SEMANTICS   0x00200000

#define AZMM_ALLOCATE_HEAP              	0x0200
#define AZMM_ALLOCATE_NO_OVERDRAFT      	0x1000
#define AZMM_ALLOCATE_REQUEST_NOZERO    	0x2000
#define AZMM_ALLOCATE_PREFER_PREZEROED		0x4000
#define AZMM_MAY_RECYCLE_BEFORE_INVALIDATE 	0x00000004

#define AZMM_FLUSH_HEAP                 	0x0001
#define AZMM_FLUSH_OVERDRAFT_ONLY       	0x0002

#define AZMM_DEFAULT_ACCOUNT            0   /* C Heap, stacks, etc */
#define AZMM_JHEAP_ACCOUNT              2
#define AZMM_GCPP_ACCOUNT               3   /* GC pause prevention */

#define AZMM_COMMITTED_FUND             0
#define AZMM_OVERDRAFT_FUND             1
#define AZMM_GCPP_FUND                  3   /* GC pause prevention */

/* az_probe_segv Return Codes */
#define VM_FAULT_SEGV_ACCERR            0x1000  /* Protection Error */
#define VM_FAULT_SEGV_MAPERR            0x2000  /* Map Error */
#define AZ_REFCOUNT_WATCHPOINT		0x80	/* Refcount Watchpoint Error */

static inline const char* azmm_get_fund_name(int id)
{
    switch(id) {
        case AZMM_COMMITTED_FUND:
            return "Committed";
            break;
        case AZMM_OVERDRAFT_FUND:
            return "Overdraft";
            break;
        case AZMM_GCPP_FUND:
            return "PausePrevention";
            break;
        default:
            return "Unknown";
            break;
    }
}

static inline const char* azmm_get_account_name(int id)
{
    switch(id) {
        case AZMM_DEFAULT_ACCOUNT:
            return "Default";
            break;
        case AZMM_JHEAP_ACCOUNT:
            return "JavaHeap";
            break;
        case AZMM_GCPP_ACCOUNT:
            return "PausePrevention";
            break;
        default:
            return "Unknown";
            break;
    }
}


/* Macros for building vectors of accounts to use with az_pmem_set_accounts_to_credit() */
#define AZMM_ACCOUNT_SHIFT 8
#define AZMM_ACCOUNT_MASK  ((1 << AZMM_ACCOUNT_SHIFT) - 1)
#define az_pmem_push_account_on_list(accounts, account) \
        (((accounts) << AZMM_ACCOUNT_SHIFT) | ((account) & AZMM_ACCOUNT_MASK))

/* Macro for iterating through a vector of accounts. */
#define az_pmem_for_each_account(i, a, accts) \
        for ( (i) = 0, (a) = (accts) & AZ_PMEM_ACCT_MASK; \
              (((a) != AZ_PMEM_ACCT_MASK) && (i) < AZMAT_NR_ACCOUNT_TYPES); \
              (i)++, (a) = ((accts) >> ((i) * AZ_PMEM_ACCT_SHIFT))& AZ_PMEM_ACCT_MASK)


extern int64_t az_mreserve(void *addr, size_t len, int flags);
extern int64_t az_mreserve_alias(void *addr, void *existing_addr);

/*
   az_mmap is the public entry point, which can be hooked for leak
   detection.

   az_mmap2 is a private unhooked entry point, and only exists to
   enable leak detection code from triggering reentrant paths.  do not
   call az_mmap2 directly.
*/
extern int64_t az_mmap2(void *addr, size_t len, int prot, int acct, int flags);
extern int64_t az_mmap(void *addr, size_t len, int prot, int acct, int flags);

extern int64_t az_munmap(void *addr, size_t len, int acct, int flags);
extern int64_t az_mremap(void *old_addr, void *new_addr, size_t len, int flags);
extern int64_t az_mprotect(void *addr, size_t len, int prot, int flags);
extern int64_t az_mcopy(void *dst_addr, void *src_addr, size_t len);
extern int64_t az_mbatch_start(void);
extern int64_t az_mbatch_remap(void *old_addr, void *new_addr,
		size_t len, int flags);
extern int64_t az_mbatch_protect(void *addr, size_t len,
		int prot, int flags);
extern int64_t az_mbatch_commit(void);
extern int64_t az_mbatch_abort(void);
extern int64_t az_tlb_invalidate(void);
extern int64_t az_munshatter(void *addr, void *force_addr, void *resource_addr,
		int prot, int acct, int flags);
extern int64_t az_mflush(int acct, int flags, size_t *allocated);

/* Serial Versions of Aznix Ioctls for use by malloc */
extern int64_t az_serial_mreserve(void *addr, size_t len, int flags);
extern int64_t az_serial_mmap(void *addr, size_t len, int prot, int acct,
		int flags);
extern int64_t az_serial_mremap(void *old_addr, void *new_addr, size_t len,
		int flags);
extern int64_t az_serial_mprotect(void *addr, size_t len, int prot, int flags);
extern int64_t az_serial_munmap(void *addr, size_t len, int acct, int flags);
extern int64_t az_serial_munreserve(void *addr, size_t len, int flags);
extern int az_is_running_on_container(void);
extern int64_t az_set_spinlock_debug(int );
extern unsigned long az_get_fault_addr(unsigned int, unsigned long *);

/* Aznix Physical Memory Accounting system calls */
/* .ko ioctl interface */
extern int az_select_partition(int partition);
extern int64_t az_munreserve(void *addr, size_t len, int flags);
extern int64_t az_zmd_api_version(void);
extern int64_t az_zst_is_coredumping(void);
extern int64_t az_soft_lockup(void);
extern int64_t az_zst_version(char *, size_t *);
extern int64_t az_pmem_set_maximum(pid_t pid, size_t size);
extern int64_t az_pmem_set_account_maximum(pid_t pid,
        unsigned long acct, size_t size);
extern int64_t az_pmem_fund_account(pid_t pid,
        unsigned long acct, size_t size);
extern int64_t az_pmem_account_transfer(pid_t pid,
        unsigned long dst, unsigned long src, size_t size);
extern int64_t az_pmem_get_account_stats(pid_t pid, void * statsp,
		size_t * sizep);
extern int64_t az_pmem_get_account_usage(pid_t pid, void * statsp,
		size_t * sizep);
extern int64_t az_pmem_get_prezero_stats(pid_t pid, void * statsp,
		size_t * sizep);
extern int64_t az_pmem_get_fund_stats(void * statsp,
        size_t * sizep);
extern int64_t az_pmem_set_account_funds(pid_t pid,
        unsigned long acct, unsigned long commit, unsigned long overdraft);
extern int64_t az_pmem_set_accounts_to_credit(pid_t pid,
        unsigned long acct, unsigned long accts_to_credit);
extern int64_t az_pmem_fund_transfer(unsigned long dst,
        unsigned long src, size_t size);
extern int64_t az_pmem_fund_limit_transfer(unsigned long dst,
        unsigned long src, size_t limit);
extern int64_t az_pmem_reserve_pages(unsigned long nr_pages,
		unsigned int nodemask);
extern int64_t az_pmem_unreserve_pages(unsigned long nr_pages,
		long fund_nr);
extern int64_t az_pmem_reset_account_watermarks(unsigned long acct);
extern int64_t az_pmem_prezero(size_t bytes_to_prezero, size_t *prezeroed_sizep);
extern int az_pmem_prefill_scrubbed_freelist(pid_t pid);
extern int az_pmem_prefill_scrubbed_freelist_chunk(pid_t pid, unsigned int);
extern int64_t az_pmem_subsys_available(void);
extern int64_t az_pmem_subsys_initialized(void);
extern int64_t az_mprobe(pid_t pid, void *addr, int flags, int *refcount);
extern long az_mprobe2(pid_t pid, void *addr, int flags,
		int *refcntp, int64_t *p2p_pfnp);
extern int64_t az_probe_segv(unsigned long address);
extern int az_get_loads(long *global, long *pid, long *cores, size_t cores_len);
extern int az_set_elastic(unsigned int nodemask);
extern int az_pmem_set_device_limit(unsigned long size);
extern int az_set_lock_flag(void *addr, size_t len);
extern int az_get_elastic(void); 
extern int az_verify_vmemmap_sections(void);
extern int az_count_empty_l1_ptes(size_t *);

/* Page table entry probe functions */
#include <sys/mman.h>
#define _PAGE_BIT_PRESENT   0   /* is present */
#define _PAGE_BIT_RW        1   /* writeable */
#define _PAGE_BIT_USER      2   /* userspace addressable */
#define _PAGE_BIT_PSE       7   /* 2MB page */
#define _PAGE_BIT_NX        63  /* No execute: only valid after cpuid check */

#define _PAGE_PRESENT   (1ULL << _PAGE_BIT_PRESENT)
#define _PAGE_RW        (1ULL << _PAGE_BIT_RW)
#define _PAGE_USER      (1ULL << _PAGE_BIT_USER)
#define _PAGE_PSE       (1ULL << _PAGE_BIT_PSE)
#define _PAGE_NX        (1ULL << _PAGE_BIT_NX)

#define PROT_MAPPED     0x10000
#define PROT_LARGE      0x20000
#define PROT_P2P        0x40000
#define PROT_INVALID    0x80000
#define pmd_prot(a)     (az_pte_prot(a) | PROT_LARGE)

/*XXX PROT_P2P Seems to be wrong. Need to Correct it. */
static inline int az_pte_prot(unsigned long pte)
{
    int prot = 0;
    if (pte & _PAGE_PRESENT)
        prot |= PROT_MAPPED;
    if (pte && !(pte & _PAGE_PRESENT))
        prot |= PROT_P2P;
    if (pte & _PAGE_USER)
        prot |= PROT_READ;
    if (pte & (_PAGE_USER | _PAGE_RW))
        prot |= PROT_WRITE;
    if ((pte & _PAGE_USER) && !(pte & _PAGE_NX))
        prot |= PROT_EXEC;
    return prot;
}

static inline int pmd_large_virtual(unsigned long pmd)
{
    return (pmd & _PAGE_PSE);
}

static inline int az_page_prot(pid_t pid, void *addr, int flags)
{
    unsigned long map;

    int refcount;

    map = az_mprobe(pid, addr, flags, &refcount);
    if ((long)map == (long)-1  && errno == EINVAL)
        return PROT_INVALID;
    if ((long)map == (long)-1)
        return 0;
    if (pmd_large_virtual(map))
        return pmd_prot(map);
    return az_pte_prot(map);
}

static inline int az_page_refcount(void *addr, int flags)
{
    int refcount;
    int map = az_mprobe(0, addr, flags, &refcount);
    if (map == -1)
	    return -1;
    return refcount;
}

#define az_large_page(addr, flags) \
    (((az_page_prot(0, (addr), (flags))) & \
      (PROT_LARGE | PROT_MAPPED)) == (PROT_LARGE | PROT_MAPPED))

#define az_page_writable(addr, flags) \
    (((az_page_prot(0, (addr), (flags))) & PROT_WRITE))

#define az_page_present(addr, flags) \
    (((az_page_prot(0, (addr), (flags))) & PROT_MAPPED))

#define az_enable_page_watchpoint(addr) \
	(az_mprobe(0, addr, AZMM_REFCOUNT_WATCHPOINT_IFNOT512, NULL))
#define az_disable_page_watchpoint(addr)	\
	(az_mprobe(0, addr, AZMM_DISABLE_REFCOUNT_WATCHPOINT, NULL))

#define az_return_first_empty_l1_pte_addr(addr, len)	\
	(az_mprobe2(len, addr, AZMM_MPROBE_RETURN_EMPTY_L1_ADDR, NULL, NULL))	
#define az_system_zing_memory_policy_is_reserve_at_config !az_get_elastic 

#ifdef __cplusplus
} // extern "C"
#endif // __cplusplus
  
#endif // _OS_AZ_MEMORY_H_
