/* az_mm_ioctl.h - Azul "I/O" definitions and access functions
 *
 * Copyright Azul Systems, 2010
 * Author Bill Gallmeister <bog@azulsystems.com>
 *
 * Copyright 2010 Azul Systems, Inc.  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * This code is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License version 2 only, as published by 
 * the Free Software Foundation. 
 * 
 * This code is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE.  See the GNU General Public License version 2 for  more
 * details (a copy is included in the LICENSE file that accompanied this code).
 * 
 * You should have received a copy of the GNU General Public License version 2 
 * along with this work; if not, write to the Free Software Foundation,Inc., 
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 * 
 * Please contact Azul Systems, Inc., 1600 Plymouth Street, Mountain View, 
 * CA 94043 USA, or visit www.azulsystems.com if you need additional information 
 * or have any questions.
 *
 * Definitions herein are for the Azul "syscalls masquerading as
 * IOCTLs" part of the Azul MM kernel module.
 */

#ifndef _AZ_MM_IOCTL_H
#define _AZ_MM_IOCTL_H

/*
 * /dev/azul ioctls and arguments for same
 */

struct az_munreserve_args {
	unsigned long addr;
	size_t len;
	int flags;
};
#define IOC_AZ_MUNRESERVE 	_IOR('z', 0x87, struct az_munreserve_args)

struct az_mmap_args {
	unsigned long addr;
	size_t len;
	int prot;
	int acct;
	int flags;
};
#define IOC_AZ_MMAP 		_IOR('z', 0x88, struct az_mmap_args)

struct az_munmap_args {
	unsigned long addr;
	size_t len;
	int acct;
	int flags;
};
#define IOC_AZ_MUNMAP 		_IOR('z', 0x89, struct az_munmap_args)

struct az_mapping_len_args {
	void *addr;
	size_t *len;
};
#define IOC_AZ_MAPPING_LEN 	_IOR('z', 0x8a, struct az_mapping_len_args)

struct az_mremap_args {
	unsigned long old_addr;
	unsigned long new_addr;
	size_t len;
	int flags;
};
#define IOC_AZ_MREMAP 		_IOR('z', 0x8b, struct az_mremap_args)

struct az_mprotect_args {
	unsigned long addr;
	size_t len;
	int prot;
	int flags;
};
#define IOC_AZ_MPROTECT 	_IOR('z', 0x8c, struct az_mprotect_args)

struct az_mcopy_args {
	unsigned long dst_addr;
	unsigned long src_addr;
	size_t len;
};
#define IOC_AZ_MCOPY 		_IOR('z', 0x8d, struct az_mcopy_args)

struct az_mbatch_remap_args {
	unsigned long old_addr;
	unsigned long new_addr;
	size_t len;
	int flags;
};
#define IOC_AZ_MBATCH_REMAP 	_IOR('z', 0x8e, struct az_mbatch_remap_args)

struct az_mbatch_protect_args {
	unsigned long addr;
	size_t len;
	int prot;
	int flags;
};
#define IOC_AZ_MBATCH_PROTECT 	_IOR('z', 0x8f, struct az_mbatch_protect_args)

struct az_munshatter_args {
	unsigned long addr;
	unsigned long force_addr;
	unsigned long resource_addr;
	int prot;
	int acct;
	int flags;
};
#define IOC_AZ_MUNSHATTER 	_IOR('z', 0x90, struct az_munshatter_args)

struct az_mprobe_args {
	pid_t pid;
	unsigned long addr;
	int flags;
	int *refcntp;
	int64_t *addrp;
	int64_t *p2p_pfnp;
};
#define IOC_AZ_MPROBE 		_IOR('z', 0x91, struct az_mprobe_args)

struct az_mflush_args {
	int acct;
	int flags;
        size_t *allocatedp;
};

#define IOC_AZ_MFLUSH 		_IOR('z', 0x92, struct az_mflush_args)

#define IOC_AZ_MBATCH_START	_IO('z', 0x94)
#define IOC_AZ_MBATCH_COMMIT	_IO('z', 0x95)
#define IOC_AZ_MBATCH_ABORT	_IO('z', 0x96)
#define IOC_AZ_TLB_INVALIDATE	_IO('z', 0x97)


struct az_pmem_set_maximum_args {
    pid_t pid;
    size_t size;
};
#define IOC_AZ_PMEM_SET_MAXIMUM	_IOR('z', 0x98, struct az_pmem_set_maximum_args)

struct az_pmem_set_account_maximum_args {
    pid_t pid;
    unsigned long acct;
    size_t size;
};
#define IOC_AZ_PMEM_SET_ACCOUNT_MAXIMUM		\
			_IOR('z', 0x99, struct az_pmem_set_account_maximum_args)

struct az_pmem_fund_account_args {
    pid_t pid;
    unsigned long acct;
    size_t size;
};
#define IOC_AZ_PMEM_FUND_ACCOUNT		\
			_IOR('z', 0x9a, struct az_pmem_fund_account_args)

struct az_pmem_account_transfer_args {
    pid_t pid;
    unsigned long dst;
    unsigned long src;
    size_t size;
};
#define IOC_AZ_PMEM_ACCOUNT_TRANSFER		\
		_IOR('z', 0x9b, struct az_pmem_account_transfer_args)

struct az_pmem_get_account_stats_args {
    pid_t pid;
    void *statsp;
    size_t *sizep;
};
#define IOC_AZ_PMEM_GET_ACCOUNT_STATS	\
		_IOR('z', 0x9c, struct az_pmem_get_account_stats_args)

struct az_pmem_get_account_usage_args {
    pid_t pid;
    void *statsp;
    size_t *sizep;
    size_t *acct_sizep;
};
#define IOC_AZ_PMEM_GET_ACCOUNT_USAGE		\
		_IOR('z', 0xa9, struct az_pmem_get_account_usage_args)


struct az_pmem_get_fund_stats_args {
    void *statsp;
    size_t *sizep;
};
#define IOC_AZ_PMEM_GET_FUND_STATS		\
		_IOR('z', 0x9d, struct az_pmem_get_fund_stats_args)

struct az_pmem_set_account_funds_args {
    pid_t pid;
    unsigned long acct;
    unsigned long commit;
    unsigned long overdraft;
};
#define IOC_AZ_PMEM_SET_ACCOUNT_FUNDS		\
		_IOR('z', 0x9e, struct az_pmem_set_account_funds_args)

struct az_pmem_set_accounts_to_credit_args {
    pid_t pid;
    unsigned long acct;
    unsigned long accts_to_credit;
};
#define IOC_AZ_PMEM_SET_ACCOUNTS_TO_CREDIT	\
		_IOR('z', 0x9f, struct az_pmem_set_accounts_to_credit_args)

struct az_pmem_fund_transfer_args {
    unsigned long dst;
    unsigned long src;
    size_t size;
};
#define IOC_AZ_PMEM_FUND_TRANSFER	\
		_IOR('z', 0xa0, struct az_pmem_fund_transfer_args)

struct az_pmem_reserve_pages_args {
    unsigned long nr_pages;
    unsigned int nodemask;
};
#define IOC_AZ_PMEM_RESERVE_PAGES	\
		_IOR('z', 0xa1, struct az_pmem_reserve_pages_args)

struct az_pmem_unreserve_pages_args {
    unsigned long nr_pages;
    long fund_nr;
};
#define IOC_AZ_PMEM_UNRESERVE_PAGES	\
		_IOR('z', 0xa2, struct az_pmem_unreserve_pages_args)

struct az_pmem_reset_account_watermarks_args {
	unsigned long acct;
};
#define IOC_AZ_PMEM_RESET_ACCOUNT_WATERMARKS	\
		_IOR('z', 0xa4, struct az_pmem_reset_account_watermarks_args)

struct az_probe_segv_args {
	unsigned long address;
};
#define IOC_AZ_PROBE_SEGV	_IOR('z', 0xa5, struct az_probe_segv_args)

struct az_pmem_prefill_scrubbed_freelist_old_args {
	pid_t pid;
};
#define IOC_AZ_PMEM_PREFILL_SCRUBBED_FREELIST_OLD   \
		_IOR('z', 0xa6, struct az_pmem_prefill_scrubbed_freelist_old_args)

struct az_pmem_prefill_scrubbed_freelist_args {
	pid_t pid;
	unsigned int nr_pages;
};
#define IOC_AZ_PMEM_PREFILL_SCRUBBED_FREELIST   \
		_IOR('z', 0xa6, struct az_pmem_prefill_scrubbed_freelist_args)

struct az_zst_version_args {
	char *versionp;
	size_t *sizep;
};
#define IOC_AZ_ZST_VERSION  _IOR('z', 0xa7, struct az_zst_version_args)

#define IOC_AZ_API_VERSION	_IO('z', 0xa8)

#define IOC_AZ_IS_COREDUMPING      _IO('z', 0xb0)

struct az_set_spinlock_debug_args {
	int on;
};
#define IOC_AZ_SET_SPINLOCK_DEBUG  \
	_IOR('z', 0xb1, struct az_set_spinlock_debug_args)

struct az_get_loads_args {
	long *global;
	long *pid;
	long *cores;
	size_t cores_len;
};
#define IOC_AZ_GET_LOADS _IOR('z', 0xbb, struct az_get_loads_args)

struct az_set_elastic_args {
	unsigned int nodemask;
};
#define IOC_AZ_SET_ELASTIC _IOR('z', 0xaa, struct az_set_elastic_args)

struct az_pmem_fund_limit_transfer_args {
    unsigned long dst;
    unsigned long src;
    size_t limit;
};
#define IOC_AZ_PMEM_FUND_LIMIT_TRANSFER	\
		_IOR('z', 0xab, struct az_pmem_fund_limit_transfer_args)

struct az_pmem_set_device_limit_args {
	unsigned long size;
};
#define IOC_AZ_PMEM_SET_DEVICE_LIMIT   \
	_IOR('z', 0xac, struct az_pmem_set_device_limit_args)

struct az_set_lock_flag_args {
	unsigned long addr;
	size_t len;
};
#define IOC_AZ_SET_LOCK_FLAG    _IOR('z', 0xad, struct az_set_lock_flag_args)

#define IOC_AZ_SOFT_LOCKUP      _IO('z', 0xae)
#define IOC_AZ_GET_ELASTIC      _IO('z', 0xaf)
#define IOC_AZ_VERIFY_VMEMMAP   _IO('z', 0xb2)

struct az_get_fault_addr_args {
	unsigned int pid;
	unsigned long *addrp;
};
#define IOC_AZ_GET_FAULT_ADDR	_IOR('z', 0xb3, struct az_get_fault_addr_args)

struct az_count_empty_l1_ptes_args {
	size_t *sizep;
};
#define IOC_AZ_COUNT_EMPTY_L1_PTES	_IOR('z', 0xb4, struct az_count_empty_l1_ptes_args) 

struct az_pmem_prezero_args {
	size_t bytes_to_prezero;
	size_t *prezeroed_sizep;
};
#define IOC_AZ_PMEM_PREZERO          _IOR('z', 0xb5, struct az_pmem_prezero_args)

struct az_pmem_get_prezero_stats_args {
	pid_t pid;
	void *statsp;
	size_t *sizep;
};
#define IOC_AZ_PMEM_GET_PREZERO_STATS          \
	_IOR('z', 0xb6, struct az_pmem_get_prezero_stats_args)


#endif /* _AZ_MM_IOCTL_H */
