#ifndef _AZPROF_ELF_HPP
#define _AZPROF_ELF_HPP

#include <azprof/azprof_debug.hpp>
#include <azprof/azprof_util.hpp>

namespace azprof {

class Elf {
public:
    Elf();
    ~Elf();

    // Imports the contents of an ELF file.
    void import(const char *from, Result&);

    // Find information about a symbol give its address.
    int sym_by_addr(
        uint64_t addr,
        const char **name = NULL,
        uint64_t *base = NULL,
        size_t *size = NULL
    ) const;

    // Copies out a portion of the executable.
    int read(char *dst, uint64_t src, size_t len) const;

private:
    static uint16_t nop2(uint16_t);
    static uint32_t nop4(uint32_t);
    static uint64_t nop8(uint64_t);
    static uint16_t swap2(uint16_t);
    static uint32_t swap4(uint32_t);
    static uint64_t swap8(uint64_t);

    static int cmp(const void*, const void*);

    struct Symbol {
        size_t name;
        uint64_t addr;
    };

    struct Segment {
        uint64_t addr;
        uint64_t size;
        char *data;
    };

    // Loaded symbols sorted by address.
    Symbol *_syms;
    size_t _symlen;

    // Buffer used to store symbol names.
    char *_strs;
    size_t _strlen;

    // Loaded segments sorted by address.
    Segment *_segs;
    size_t _seglen;
};

}; // namespace azprof

#endif // _AZPROF_ELF_HPP
