#ifndef _AZPROF_MEMORY_HPP
#define _AZPROF_MEMORY_HPP

#include "azprof_cpu.hpp"
#include "azprof_os.hpp"
#include <stddef.h>
#include <sys/types.h>

namespace azprof {

// Wrappers around malloc/free and friends for debugging by trashing memory
// after allocating or freeing it to try to detect uses of uninitialized or
// freed memory.
class Memory {
public:
    enum {
        KB = 1024,
        MB = 1024*KB,
        GB = 1024*MB
    };

    static void* alloc(size_t sz);
    static void* alloc_aligned(size_t sz, size_t align, void **unaligned);
    static void* calloc(size_t len, size_t memsz);
    static void* calloc_aligned(size_t len, size_t memsz, size_t align,
        void **unaligned);
    static void* realloc(void *ptr, size_t newsz);
    static void* realloc(void *ptr, size_t oldsz, size_t newsz);
    static void free(void *ptr);
    static void free(void *ptr, size_t sz);
    static void trash(void *ptr, size_t sz, char x = 0x55);
    static bool is_trash(void *ptr, size_t sz, char x = 0x55);

private:
    Memory();
};

} // namespace azprof

#endif // _AZPROF_MEMORY_HPP
