#ifndef _AZPROF_WEB_HPP
#define _AZPROF_WEB_HPP

#include "azprof_io.hpp"
#include "azprof_memory.hpp"
#include "azprof_security.hpp"
#include "azprof_servlets.hpp"

#include <os/utilities.h>

#ifndef AZPROF_NO_JNI
#include <jni.h>
#endif

#include <limits.h>


////////////////////////////////////////////////////////////////////////////////
//
// Features List for determining what (based on user type) users may access.
//
// This mechanism is used to display the "Features" list within the "Configuration"
// tab of the Real Time Performance Monitor (ZVision) browser display.  There
// are two Java Script Template pages associated with this feature, features.jst
// and features_update.jst.  The former is accessed whenever the Features tab is
// chosen from the broswer, the latter is accessed whenever any portion of the
// Features page is modified, and contains all the logic for handling the user
// confirmation process.
//
//
//
//
////////////////////////////////////////////////////////////////////////////////

// HACK ALERT!! - This isn't the right place for this.  We put it here as a temporary
// means of allowing both this logic and the artaQuery.cpp's handle_features...() methods
// to share this code.

////////////////////////////////////////////////////////////////////////////////////////
//  XML Strings.  NOTE: This list of defines should ultimately be part of a larger
//  group of defines for all strings used either within the XML script or as strings
//  displayed on the browser.
////////////////////////////////////////////////////////////////////////////////////////

// These defines are all XML data strings used by the /config/features page.
#define FEATURE_TITLES_STR          "titles"
#define FEATURE_TITLE_STR           "title"
#define FEATURE_USERTITLE_STR       "usertitle"
#define FEATURE_FEATURES_STR        "features"
#define FEATURE_FEATURE_STR         "feature"
#define FEATURE_ENTRY_STR           "entry"
#define FEATURE_NAME_STR            "name"
#define FEATURE_USERTYPES_STR       "usertypes"
#define FEATURE_USERTYPE_STR        "usertype"
#define FEATURE_USERTYPENAME_STR    "usertypename"
#define FEATURE_USERTYPESTATE_STR   "usertypestate"
#define FEATURE_DESC_STR            "desc"
#define FEATURE_RISK_STR            "risk"
#define FEATURE_CHECKED             "checked"
#define FEATURE_CANCEL_STR          "cancel"
#define FEATURE_UPDATE_STR          "update"
#define FEATURE_CHANGED_STR         "changed"
#define FEATURE_CONFIRM_STR         "confirm"
#define FEATURE_COMPLETE_STR        "complete"
#define FEATURE_ENABLED_STR         "enabled"
#define FEATURE_TRUE_STR            "true"
#define FEATURE_FALSE_STR           "false"
#define FEATURE_UPDATE_REQ_STR      "update_required"

// The levels of authorization we currently support.  These are the XML data strings
// we will send back to the browser.  The FEATURE_TYPE_MAX_STR should be at least as
// big as the largest string length in this list.
#define FEATURE_TYPE_ADMIN_STR      "zvadmin"
#define FEATURE_TYPE_USER_STR       "zvuser"
#define FEATURE_TYPE_SUPER_STR      "zvsuperuser"
#define FEATURE_TYPE_SPPRT_STR      "azspprt"
#define FEATURE_TYPE_AZENG_STR      "azeng"
#define FEATURE_TYPE_MAX_STR        (40)               

// These are the XML title strings we use to map to the authorization levels above.
// Make sure all of these strings are the same length, and centered within them.
#define FEATURE_TITLE_ADMIN_STR    "Admin"
#define FEATURE_TITLE_USER_STR     "User"
#define FEATURE_TITLE_SUPER_STR    "SuperUser"
#define FEATURE_TITLE_SPPRT_STR    "Azul Field Support"
#define FEATURE_TITLE_AZENG_STR    "Azul Engineering"

// Each Feature will have a name.
#define FEATURE_NAME_SPPRT_STR     "Azul Support"
#define FEATURE_NAME_BRSW_INST_STR "Browse Instance"
#define FEATURE_NAME_BRSW_CLSS_STR "Browse Class"
#define FEATURE_NAME_BRSW_MTHD_STR "Browse Method"
#define FEATURE_NAME_ENV_VARS_STR  "Environment variables"
#define FEATURE_NAME_STK_TRC_STR   "Stack trace"
#define FEATURE_NAME_WEBLGCMUX_STR "Weblogic Muxer"
#define FEATURE_NAME_METATICKS_STR "Metaticks"
#define FEATURE_NAME_TOP_SAFEP_STR "Polling Opportunities"
#define FEATURE_NAME_EVNTTRCKR_STR "Event Tracker"

// For each Feature we want to briefly describe what the feature is.
#define FEATURE_DESC_SPPRT_STR     "Azul Customer Support Options."
#define FEATURE_DESC_BRSW_INST_STR "Allow users to browse a class instance."
#define FEATURE_DESC_BRSW_CLSS_STR "Allow users to browse the static class data."
#define FEATURE_DESC_BRSW_MTHD_STR "Allow users to browse a class method."
#define FEATURE_DESC_ENV_VARS_STR  "Allow users to view the Environment Variables."
#define FEATURE_DESC_STK_TRC_STR   "Allow users to use the Stack Trace subtab."
#define FEATURE_DESC_WEBLGCMUX_STR "Allow users to use the Web Logic Muxer Application."
#define FEATURE_DESC_METATICKS_STR "Allow users to view Metaticks."
#define FEATURE_DESC_TOP_SAFEP_STR "Allow users to view the Polling Opportunities entries."

// For each Feature we want to tell the user what the risks of using it are.
#define FEATURE_RISK_SPPRT_STR     "Features in this tab may cause interfere with the proper functioning of the VM and cause the system to crash. This tab should ONLY be enabled if advised by Azul Support."
#define FEATURE_RISK_BRSW_INST_STR "Turning this feature on could expose sensitive data within the object."
#define FEATURE_RISK_BRSW_CLSS_STR "Turning this feature on could expose sensitive data within the object."
#define FEATURE_RISK_BRSW_MTHD_STR "Turning this feature on could expose sensitive data within the object."
#define FEATURE_RISK_ENV_VARS_STR  "Turning this feature on could expose sensitive data."
#define FEATURE_RISK_STK_TRC_STR   "Turning this feature on could expose sensitive data."
#define FEATURE_RISK_WEBLGCMUX_STR "Turning this feature on could interfere with the proper functioning of the VM and cause the system to crash. This tab should ONLY be enabled if advised by Azul Support."
#define FEATURE_RISK_METATICKS_STR "Turning this feature on could interfere with the proper functioning of the VM and cause the system to crash. This tab should ONLY be enabled if advised by Azul Support."
#define FEATURE_RISK_TOP_SAFEP_STR "Turning this feature on could interfere with the proper functioning of the VM and cause the system to crash. This tab should ONLY be enabled if advised by Azul Support."

// These defines must match the order of the _pUserTypes & _pUserTitles table.
#define FEATURE_USER_ADMIN         (0)  // Customer Administrator.
#define FEATURE_USER_USER          (1)  // Typical Customer Usage.
#define FEATURE_USER_SUPER         (2)  // Customer Super User. (Senior Technical Staff)  
#define FEATURE_USER_SPPRT         (3)  // Azul Field Support Staff.
#define FEATURE_USER_AZENG         (4)  // Azul Engineering Staff.
#define FEATURE_MAX_USER_TYPES     (5)  // Must be 1 greater than the last FEATURE_... user offset entries above.
#define FEATURE_MAX_TITLES         (FEATURE_MAX_USER_TYPES)

// These defines must match the order of the _featureInfo table.
#define FEATURE_SPPRT_TAB          (0)
#define FEATURE_BRWS_INST          (1)
#define FEATURE_BRWS_CLSS          (2)
#define FEATURE_BRWS_MTHD          (3)
#define FEATURE_ENV_VARS           (4)
#define FEATURE_STK_TRACE          (5)
#define FEATURE_METATICKS          (6)
#define FEATURE_TOP_SAFEP          (7)
#define FEATURE_WEBLGCMUX          (8)
#define FEATURE_MAX_ENTRIES        (9) // Must be 1 greater than the last FEATURE_... feature offset entries above.

#define FEATURE_ACTUAL_ENTRIES     ((feature_t) (sizeof(_featureInfo) / sizeof(FEATURE_INFO_ROW_T)))
#define FEATURE_ACTUAL_USER_TYPES  ((feature_t) (sizeof(_pUserTypes) / sizeof(char*)))
#define FEATURE_ACTUAL_TITLES      ((feature_t) (sizeof(_pUserTitles) / sizeof(char*))) 

// Each array dimension is an unsigned long.
typedef unsigned long              feature_t;

// Features can be assigned to the category, the 
// subcategory, or a specific element of a page.
typedef enum 
{
  FEATURE_CATEGORY,
  FEATURE_SUBCAT,
  FEATURE_ELEMENT,
} feature_usage_t;

// TODO: Feature permissions are stored in RAM, but
// could be saved out to a config file between
// JVM sessions.
typedef struct FEATURE_INFO_ROW
{
  feature_t       entryNo;                          // The FEATURE_FEATURE_* define, which must match the array offset.
  feature_usage_t usage;                            // Is this a category, subcategory, or element from an HTML page.
  const char*     pName;                            // Feature name on browser's /config/features page.
  bool            userType[FEATURE_MAX_USER_TYPES]; // Which authorization-type {user, admin, etc.} checkboxes are checked.
  const char*     pDesc;                            // Description of feature on browser's /config/features page.
  const char*     pRisk;                            // Description of risk on browser's /config/features page.
} FEATURE_INFO_ROW_T;

typedef struct FEATURE_INFO
{
  FEATURE_INFO_ROW_T entry;
} FEATURE_INFO_T;

// Between when a Feature is modified, and when
// the user confirms that modification, we may
// need to be able to restore the prior state.
typedef struct PRIOR_FEATURE
{
  bool           userType[FEATURE_MAX_USER_TYPES]; // Which authorization-type {user, admin, etc.} checkboxes are checked.
} PRIOR_FEATURE_T;

// APIs of various features.
extern feature_t    FeatureMaxUserTypes();
extern feature_t    FeatureActualUserTypes();
extern feature_t    FeatureMaxTitles();
extern feature_t    FeaturActualTitles();
extern const char*  FeatureUserTypeGet(feature_t col);
extern const char*  FeatureUserTitleGet(feature_t col);
extern feature_t    FeatureMaxEntries();
extern feature_t    FeatureActualEntries();
extern bool         FeaturePermGet(feature_t row, feature_t col);
extern void         FeaturePermSet(feature_t row, feature_t col, bool val);
extern feature_t    FeatureEntryNoGet(feature_t row);
extern const char*  FeatureNameGet(feature_t row);
extern const char*  FeatureDescGet(feature_t row);
extern const char*  FeatureRiskGet(feature_t row);
extern const char*  FeaturePermCheckedStr(feature_t row, feature_t col);
extern bool         FeatureRestricted(const char* name, feature_usage_t use, azprof::Request* req);
extern feature_t    FeatureConvertLevelToUserType(int level);



namespace azprof {

    class WebServer;
    class WebServlet;
    class FileServlet;



////////////////////////////////////////////////////////////////////////////////
// Pre-declared core servlets
////////////////////////////////////////////////////////////////////////////////

class SecurityServlet : public Servlet {
public:
    SecurityServlet(WebServer*, ServletSet*);

    const WebServer* web_server() const {return _websvr;}
    const ServletSet* servlet_set() const {return _servlet_set;}

    virtual void pre_service(Request*, Response*);
    virtual void service(Request*, Response*);
    virtual void post_service(Request*, Response*);
private:
    WebServer *_websvr;
    ServletSet *_servlet_set;
};

////////////////////////////////////////////////////////////////////////////////
// WebServer
////////////////////////////////////////////////////////////////////////////////

class WebServer {
public:
    class Addr {
    public:
        Addr() : _port(-1) {
            strlcpy(_interface, "0.0.0.0", sizeof(_interface));
        }
        Addr(const char *__interface, int __port) : _port(__port) {
            strlcpy(_interface, __interface, sizeof(_interface));
        }
        Addr(const Addr& addr) : _port(addr._port) {
            strlcpy(_interface, addr._interface, sizeof(_interface));
        }
        Addr& operator=(const Addr& addr) {
            strlcpy(_interface, addr._interface, sizeof(_interface));
            _port = addr._port;
            return *this;
        }

        static Addr none() {return Addr();}

        const char* interface() const {return _interface;}
        void set_interface(const char *s) {
            strlcpy(_interface, s, sizeof(_interface));
        }
        int port() const {return _port;}
        void set_port(int k) {_port = k;}

    private:
        char _interface[128];
        int _port;
    };

    class Config {
    public:
        Config();

        const char* zvm_version() const {return _zvm_version;}
        void set_zvm_version(const char *s) {
            strlcpy(_zvm_version, s, sizeof(_zvm_version));
        }

        const char* name() const {return _name;}
        void set_name(const char *s) {strlcpy(_name, s, sizeof(_name));}

        const char* abbreviation() const {return _abbrev;}
        void set_abbreviation(const char *s) {
            strlcpy(_abbrev, s, sizeof(_abbrev));
        }

        const Addr& addr1() const {return _addr1;}
        void set_addr1(const Addr& a) {_addr1 = a;}

        const Addr& addr2() const {return _addr2;}
        void set_addr2(const Addr& a) {_addr2 = a;}

        const char* authentication() const {return _authen;}
        void set_authentication(const char *s) {
            strlcpy(_authen, s, sizeof(_authen));
        }

        const char* authorization() const {return _author;}
        void set_authorization(const char *s) {
            strlcpy(_author, s, sizeof(_author));
        }

        const char* log() const {return _log;}
        void set_log(const char *s) {strlcpy(_log, s, sizeof(_log));}

        const char* resource_dir() const {return _rc_dir;}
        void set_resource_dir(const char *s) {
            strlcpy(_rc_dir, s, sizeof(_rc_dir));
        }

        uint64_t debug_flags() const {return _dbg_flags;}
        void set_debug_flags(uint64_t m) {_dbg_flags = m;}

    private:
        char _zvm_version[64];
        char _name[64];
        char _abbrev[8];
        Addr _addr1;
        Addr _addr2;
        char _authen[256];
        char _author[256];
        char _log[PATH_MAX];
        char _rc_dir[PATH_MAX];
        uint64_t _mem_limit;
        uint64_t _dbg_flags;
    };

    class ContextMark {
    public:
        ContextMark(WebServer *websvr);
    private:
        LogMark _log_mark;
        DebugFlagMark _debug_flag_mark;
    };

    ~WebServer();

    static WebServer* init(const Config&, Result&);
    static WebServer* self() {return _self;}
    static WebServer* create(const Config&, Result&);

    static void begin_xml_response(Request*, Response*, const char *xsl_name);
    static void end_xml_response(Request*, Response*);

    Log* log() {return &_log;}
    const char* zvm_version() const {return _zvm_version;}
    const char* name() const {return _name;}
    const char* abbreviation() const {return _abbrev;}
    const char* resource_dir() const {return _rc_dir;}
    uint64_t debug_flags() const {return _dbg_flags;}

    void set_detach_callback(Server::DetachCallback, void*);

    bool add_servlet(const char *path, Servlet *servlet);
    bool add_servlet(
        const char *path,
        const char *category,
        const char *subcategory,
        Servlet *servlet
    );
    bool add_web_servlet(
        const char *path,
        const char *category,
        const char *subcategory,
        const char *xsl_name,
        Servlet *servlet
    );

    void to_xml(Request*, Response*);
    void service_details(Request*, Response*);
    void service_security(Request*, Response*, ServletSet*);
    void service_login(Request*, Response*);
    void service_logout(Request*, Response*);
    void begin_response(Request*, Response*, WebServlet*);
    void end_response(Request*, Response*, WebServlet*);

    void start(
        const Addr& addr1, const Addr& addr2, const char *authentication,
        const char *authorization, bool reattempt_bind, Result&
    );
    void stop(Result&);

    void start(
        const char *interface, int port, const char *authentication,
        const char *authorization, bool reattempt_bind, Result&
    );
    void done_loading();

    // Return the port number for the HTTP server
    // If "any" is specified then the port is assigned on start-up but
    // the new value is not propagated to the _addr1 field in the WebServer.
    // This preserves the original value of the port number. We are using
    // only http_server1 (and the corresponding first of the two Addr fields, 
    // _addr1), so let's only add what's needed to this class's interface.
    int http_server1_port() {return _http_server1.port();}

private:
    enum {
        LOADED = 0x1,
    };

    class Subcategory {
    public:
        enum {
            MAX_COUNT = 32
        };

        Subcategory();

        const char* name() const {return _name;}
        const char* path() const {return _path;}
        const Servlet* servlet() const {return _servlet;}
        bool requires_azul_engineer_access() const {
            return servlet()->requires_azul_engineer_access();
        }

        void init(const char *name, const char *path, const Servlet *servlet);

    private:
        const char *_name;
        const char *_path;
        const Servlet *_servlet;
    };

    class Category {
    public:
        enum {
            MAX_COUNT = 16
        };

        Category();

        const char* name() const {return _name;}
        const Subcategory& subcategory_at(size_t i) const {
            azprof_assert(i < _subcategory_count);
            return _subcategories[i];
        }
        size_t subcategory_count() const {return _subcategory_count;}
        bool requires_azul_engineer_access() const;

        void init(const char *name);
        void add_subcategory(
            const char *name, const char *path, const Servlet *servlet
        );

    private:
        const char *_name;
        Subcategory _subcategories[Subcategory::MAX_COUNT];
        size_t _subcategory_count;
    };
    
    class StateMayChange {
    public:
        StateMayChange(WebServer*);
        ~StateMayChange();
    private:
        WebServer *_server;
    };

    friend class WebServer::StateMayChange;
    
    static WebServer *_self;

    WebServer(const Config&);

    void init();
    void state_may_have_changed();

    Log _log;
    char _zvm_version[64];
    char _name[64];
    char _abbrev[8];
    char _rc_dir[PATH_MAX];
    uint64_t _dbg_flags;
    uint64_t _start_time;

    Mutex _state_mutex;
    int _flags;
    Authenticator _authenticator;
    HttpServer _http_server1;
    HttpServer _http_server2;
    AZPROF_OPENSSL_ONLY(HttpsServer _https_server1);
    AZPROF_OPENSSL_ONLY(HttpsServer _https_server2);
    Server::Protocol _protocol;
    Addr _addr1;
    Addr _addr2;

    Mutex _servlets_mutex;
    ServletSet _servlet_set;
    Category _categories[Category::MAX_COUNT];
    size_t _category_count;

    SecurityServlet _security_servlet;
};

////////////////////////////////////////////////////////////////////////////////
// HTTP(S) server state servlets
////////////////////////////////////////////////////////////////////////////////
class DetailsServlet : public Servlet {
public:
    DetailsServlet(WebServer*);
    virtual void service(Request*, Response*);
private:
    WebServer *_websvr;
};

////////////////////////////////////////////////////////////////////////////////
// Session servlets
////////////////////////////////////////////////////////////////////////////////

class ProcessLoginServlet : public Servlet {
public:
    ProcessLoginServlet(WebServer*);
    virtual void service(Request*, Response*);
private:
    WebServer *_websvr;
};

class ProcessLogoutServlet : public Servlet {
public:
    ProcessLogoutServlet(WebServer*);
    virtual void service(Request*, Response*);
private:
    WebServer *_websvr;
};

class LoginServlet : public Servlet {
public:
    LoginServlet(WebServer*);
    virtual void service(Request*, Response*);
private:
    WebServer *_websvr;
};

////////////////////////////////////////////////////////////////////////////////
// WebServlet
////////////////////////////////////////////////////////////////////////////////

class WebServlet : public Servlet {
public:
    static void err(Response *res, const char *fmt, ...);

    WebServlet(
        WebServer *websvr, const char *category, const char *subcategory,
        const char *xsl_name, Servlet *servlet
    );

    WebServer* websvr() const {return _websvr;}
    const char* category() const {return _category;}
    const char* subcategory() const {return _subcategory;}
    const char* xsl_name() const {return _xsl_name;}
    Servlet* servlet() {return _servlet;}

    virtual void pre_service(Request*, Response*);
    virtual void service(Request*, Response*);
    virtual void post_service(Request*, Response*);

private:
    WebServer *_websvr;
    const char *_category;
    const char *_subcategory;
    const char *_xsl_name;
    Servlet *_servlet;
};

////////////////////////////////////////////////////////////////////////////////
// JNI
////////////////////////////////////////////////////////////////////////////////

#ifndef AZPROF_NO_JNI

extern "C" void Java_com_azulsystems_arta_ARTA_getWebServerState0(
    JNIEnv*, jclass, jobject);
extern "C" void Java_com_azulsystems_arta_ARTA_service0(
    JNIEnv*, jclass, jstring, jobject);

#endif // ifndef AZPROF_NO_JNI

} // namespace azprof

#endif // _AZPROF_WEB_HPP
