// Copyright 2003 Azul Systems, Inc. All rights reserved.
// AZUL Systems PROPRIETARY/CONFIDENTIAL. Use is subject to license terms

// azulmmap.h - Azul Virtual Memory Map.
//
//

#ifndef _AZULMMAP_H 
#define _AZULMMAP_H  1

#ifdef __cplusplus
extern "C" {
#endif


// Azul Virtual Memmory Allocation & Layout Defines.
//
//

// TODO -
// With the current map, we have only 1G of card mark space.
// We can support only 512G of parallel gc heap space with 1G of card table space.
// Also, the code cache is limited to 768m.

// If you change memory layout for libjvm, codecache, or ooptable (anything 
// under 2gb), bump this define.  This version is record in a libazsys data 
// structure, and can be queried from the ZVM (see os.c).
#define __AZULMMAP_VERSION__CHANGE_WHEN_VMSTRUCTS_MOVE__  2

// To prevent malloc from using low virtual address space, we reserve oopTable
// and codecache regions when the process starts up and have malloc allocate 
// in higher address ranges.  

// We generate absolute references to the oop table so it needs to either be
// in the low 2GB or RIP relative to the code cache. RIP relative addressing
// doesn't support indexing, so keep in low 2GB for now.
#define __OOP_TABLE_START_ADDR__                     0x29000000ULL  // 0G  656M
#define __OOP_TABLE_END_ADDR__                       0x2FE00000ULL  // 0G  766M

// Linux memory used to store "data under 2Gb"...a larger address range is
// reserved to prevent silly ZST update requirements if we ever need more than
// a 4K page for this purpose.  
// 2Mb page alignment for this region is intentional. 
#define __DATA_UNDER_2GB_REGION_START_ADDR__         0x2FE00000ULL  // 0G  766M
#define __DATA_UNDER_2GB_PAGE_START_ADDR__           0x2FF80000ULL  // 0G  767M 512K
#define __DATA_UNDER_2GB_PAGE_END_ADDR__             0x2FF81000ULL  // 0G  767M 516K
#define __DATA_UNDER_2GB_REGION_END_ADDR__           0x30000000ULL  // 0G  768M

// Generated code needs to reside below 2G because code offsets need to fit into
// signed 32bit offsets in instructions.
#define __CODE_CACHE_START_ADDR__                    0x30000000ULL  // 0G  768M
#define __CODE_CACHE_END_ADDR__                      0x80000000ULL  // 2G

#define __JNI_BAD_MEMORY_START_ADDR__                0xc0000000ULL  // 3G
#define __JNI_BAD_MEMORY_END_ADDR__                 0x100000000ULL  // 4G

// !AZ_PROXIED (azlinux)
// ### Avoid 4GB -> 512GB ### to leave room for typical prelink conflicts.
// prelink assigns addresses just below 256Gb to system libraries, for now.

// JNIHandleBlock addresses should not exceed 44 bits of VA
#define __JNI_HANDLE_BLOCK_START_ADDR__            0x8000000000ULL  // 512G 
#define __JNI_HANDLE_BLOCK_END_ADDR__              0x8300000000ULL  // 524G

#define __SBA_AREA_START_ADDR__                    0x8080000000ULL  // 514G
#define __SBA_AREA_END_ADDR__                      0x8300000000ULL  // 524G

// Mirrors are at a fixed offset from their base address.
// In the current code naked OOPs and pageNums get mapped to the HEAP_MIRRORHEAP_OFFSET space
#define __GPGC_HEAP_MIRROR_OFFSET__              0x5A0000000000ULL // Just an offset DEFUNCT
#define __GPGC_HEAP_MIRROR_101_OFFSET__          0x0A0000000000ULL // Two bits separated by a zero bit DEFUNCT
// Once we move to complete OOPs and pageNums we will map to mirror phase spaces as shown below
#define __GPGC_HEAP_MIRROR_1001_OFFSET__         0x120000000000ULL // 001001 bit pattern at 18T

// GPGC 6 bit RPV metadata layout: 2TB phase size
// The HEAP values are informational only. The actual code synthesizes these from the phase bits.
// The MIRROR values Informational only. The actual code synthesizes these from the heap values
//  and __GPGC_HEAP_MIRROR_1001_OFFSET__
#define __GPGC_HEAP_PHASE_000001_START_ADDR__    0x020000000000ULL //  2T
#define __GPGC_HEAP_PHASE_000001_END_ADDR__      0x040000000000ULL //  4T

#define __GPGC_HEAP_PHASE_000010_START_ADDR__    0x040000000000ULL //  4T
#define __GPGC_HEAP_PHASE_000010_END_ADDR__      0x060000000000ULL //  6T

#define __GPGC_HEAP_PHASE_000100_START_ADDR__    0x080000000000ULL //  8T
#define __GPGC_HEAP_PHASE_000100_END_ADDR__      0x0A0000000000ULL // 10T

#define __GPGC_HEAP_PHASE_001000_START_ADDR__    0x100000000000ULL // 16T
#define __GPGC_HEAP_PHASE_001000_END_ADDR__      0x120000000000ULL // 18T

#define __GPGC_MIRROR_PHASE_001010_START_ADDR__  0x140000000000ULL // 20T  Complete pageNum only
#define __GPGC_MIRROR_PHASE_001010_END_ADDR__    0x160000000000ULL // 22T  Complete pageNum only

#define __GPGC_MIRROR_PHASE_001011_START_ADDR__  0x160000000000ULL // 22T  Complete pageNum only
#define __GPGC_MIRROR_PHASE_001011_END_ADDR__    0x180000000000ULL // 24T  Complete pageNum only

#define __GPGC_MIRROR_PHASE_001101_START_ADDR__  0x1A0000000000ULL // 26T  Complete pageNum only
#define __GPGC_MIRROR_PHASE_001101_END_ADDR__    0x1C0000000000ULL // 28T  Complete pageNum only

#define __GPGC_HEAP_PHASE_010000_START_ADDR__    0x200000000000ULL // 32T
#define __GPGC_HEAP_PHASE_010000_END_ADDR__      0x220000000000ULL // 34T

#define __GPGC_MIRROR_PHASE_010001_START_ADDR__  0x220000000000ULL // 34T  Complete pageNum only
#define __GPGC_MIRROR_PHASE_010001_END_ADDR__    0x240000000000ULL // 36T  Complete pageNum only

// Informational only. Some RedHat5 systems put the system libraries in this range
#define __SHARED_RHEL5_LIBRARY_RESERVE_START_ADDR__ 0x2A0000000000ULL // 42T 
#define __SHARED_RHEL5_LIBRARY_RESERVE_END_ADDR__   0x2C0000000000ULL // 44T

// The original attempt at placing heap metadata in Zing memory. Moved as METADATA to not conflict with
// the new mirror locations after the mirror was moved to avoid the RHEL5 libraries
#define __GPGC_HEAP_STRUCTURES_START_ADDR__      0x2C0000000000ULL // 44T  GPGC only DEFUNCT
#define __GPGC_HEAP_STRUCTURES_END_ADDR__        0x400000000000ULL // 64T  GPGC only DEFUNCT

#define __GPGC_MIRROR_PHASE_011001_START_ADDR__  0x320000000000ULL // 50T  Complete pageNum only
#define __GPGC_MIRROR_PHASE_011001_END_ADDR__    0x340000000000ULL // 52T  Complete pageNum only

#define __PERF_MEMORY_START_ADDR__               0x344000000000ULL // 52T 256G
#define __PERF_MEMORY_END_ADDR__                 0x348000000000ULL // 52T 512G

#define __GPGC_HEAP_PHASE_100000_START_ADDR__    0x400000000000ULL // 64T
#define __GPGC_HEAP_PHASE_100000_END_ADDR__      0x420000000000ULL // 66T

// All thread stacks will have the 0x440000000000ULL bits set:
#define __THREAD_STACK_REGION_START_ADDR__       0x440000000000ULL // 68T
#define __THREAD_STACK_REGION_END_ADDR__         0x480000000000ULL // 72T

#define __DEBUG_REMAP_REGION_START_ADDR__        0x490000000000ULL // 73T
#define __DEBUG_REMAP_REGION_END_ADDR__          0x4A0000000000ULL // 74T

#define __GPGC_MIRROR_PHASE_101001_START_ADDR__  0x520000000000ULL // 82T  Complete pageNum only
#define __GPGC_MIRROR_PHASE_101001_END_ADDR__    0x540000000000ULL // 84T  Complete pageNum only

#define __ANONYMOUS_MMAP_START_ADDR__            0x540000000000ULL // 84T
#define __ANONYMOUS_MMAP_END_ADDR__              0x550000000000ULL // 85T
                                                  
#define __GPGC_HEAP_MIRROR_START_ADDR__          0x580000000000ULL // 88T  Naked pageNum GPGC DEFUNCT
#define __GPGC_HEAP_MIRROR_END_ADDR__            0x5c0000000000ULL // 92T  Naked pageNum GPGC DEFUNCT

// This range is used starting at the END value and counting backwards for the various
// bitmaps, trampolines, etc. This means that the observed values currently start at
// 0x571180000000 and extend 0x10EE80000000 to 0x680000000000
#define __GPGC_HEAP_METADATA_START_ADDR__        0x550000000000ULL // 85T  Complete pageNum only
#define __GPGC_HEAP_METADATA_END_ADDR__          0x690000000000ULL // 105T Complete pageNum only

// Informational only. Some systems put the system libraries in this range
#define __SHARED_LIBRARY_RESERVE_START_ADDR__    0x780000000000ULL // 120T
#define __SHARED_LIBRARY_RESERVE_END_ADDR__      0x800000000000ULL // 128T


// TODO: Make this value be derived from the above invariants.
// This is a hack'n'slash for now, but if the above values change
// or the predicates in azulmmap.h change, we will need to reflect
// that in this value.  1<<46 = 0x400,0000,0000 = 64Terabytes.
#define AZUL_THREAD_STACKS_BASE_SHIFT    (42)
#define AZUL_THREAD_STACKS_SHIFTED_BASE  (0x11ULL)


#ifdef __cplusplus
} // extern "C"
#endif // __cplusplus

#endif  // _AZULMMAP_H
