/* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil -*- */
#ifndef AZSYS_LIBC_HOOKS_H
#define AZSYS_LIBC_HOOKS_H

/*
 * Copyright 2011 Azul Systems, Inc.  All rights reserved.
 * AZUL PROPRIETARY/CONFIDENTIAL.  Use is subject to license terms.
 */

typedef void (*malloc_hook_fn) (size_t bytes, void* addr);
typedef void (*free_hook_fn) (void* addr);
typedef void (*calloc_hook_fn) (size_t nmemb, size_t size, void* addr);
typedef void (*realloc_hook_fn) (void* ptr, size_t size, void* addr);
typedef void (*mmap_hook_fn) (size_t length, void** addr);
typedef void (*munmap_hook_fn) (size_t length, void* addr);
typedef void (*mremap_hook_fn) (size_t new_size, void* old_address, void* new_address, int flags);
typedef void (*dlclose_hook_fn) (void* handle);
typedef void (*pthread_create_hook_fn) (pthread_t* thread, const pthread_attr_t* attr, void* start_function, void* start_param);
typedef void (*pthread_exit_hook_fn) (void* ret);
typedef void (*fork_hook_fn)(void);

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Register a callback to be called after libc's malloc is called. The
 * callback is called with the size of the request and the address
 * returned by malloc. Only one callback can be active at a time.
 */
void register_malloc_hook(malloc_hook_fn malloc_callback);
/**
 * Register a callback to be called after libc's free is called. The
 * callback is called with the address passed to free. Only one
 * callback can be active at a time.
 */
void register_free_hook(free_hook_fn free_callback);
/**
 * Register a callback to be called after libc's calloc is called. The
 * callback is called with the member count, size of member, and the
 * address returned by calloc. Only one callback can be active at a
 * time.
 */
void register_calloc_hook(calloc_hook_fn calloc_callback);
/**
 * Register a callback to be called after libc's realloc is
 * called. The callback is called with the address to be reallocated,
 * the size of the request and the address returned by realloc. Only
 * one callback can be active at a time.
 */
void register_realloc_hook(realloc_hook_fn realloc_callback);

void register_mmap_hook(mmap_hook_fn mmap_callback);
void register_munmap_hook(munmap_hook_fn munmap_callback);
void register_mremap_hook(mremap_hook_fn mremap_callback);

void register_dlclose_hook(dlclose_hook_fn dlclose_callback);

void register_pthread_create_hook(pthread_create_hook_fn pthread_create_callback);

void register_pthread_exit_hook(pthread_exit_hook_fn pthread_exit_callback);

void register_fork_hook(fork_hook_fn fork_callback);

void set_tick_profiler_enabled(bool);
void set_arta_timer_signal(int);

#ifdef __cplusplus
}
#endif



#endif
