// Copyright 2003-2010 Azul Systems, Inc. All rights reserved.
// AZUL Systems PROPRIETARY/CONFIDENTIAL. Use is subject to license terms

// mutex.h - Azul system services mutex and condition variable functions.

#ifndef _OS_MUTEX_H_
#define _OS_MUTEX_H_ 1

#include <pthread.h>
#include <errno.h>
#include <sys/time.h>
#include <os/types.h>
#include <os/errors.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef struct timespec timespec_t;
typedef pthread_mutex_t mutex_t;
typedef pthread_cond_t condition_t;

#define MUTEX_TRUE	1
#define MUTEX_FALSE	0

static inline sys_return_t mutex_init(mutex_t *this_mutex)
{ 
    return (sys_return_t)pthread_mutex_init(this_mutex, NULL);
}

static inline sys_return_t mutex_destroy(mutex_t *this_mutex)
{
    return (sys_return_t)pthread_mutex_destroy(this_mutex);
}

static inline uint64_t mutex_trylock(mutex_t *this_mutex)
{
    return (pthread_mutex_trylock(this_mutex) == 0 ? MUTEX_TRUE : MUTEX_FALSE);
}

static inline sys_return_t mutex_lock(mutex_t *this_mutex)
{
    return (sys_return_t)pthread_mutex_lock(this_mutex);
}

static inline sys_return_t mutex_unlock(mutex_t *this_mutex)
{
    return (sys_return_t)pthread_mutex_unlock(this_mutex);
}

static inline int mutex_is_mine(mutex_t *this_mutex)
{
    return (pthread_equal(pthread_self(), this_mutex->__data.__owner) != 0 ? MUTEX_TRUE : MUTEX_FALSE);
}

static inline sys_return_t condition_init(condition_t *this_cond)
{
    return (sys_return_t)pthread_cond_init(this_cond, NULL);
}
static inline sys_return_t condition_destroy(condition_t *this_cond)
{
    return (sys_return_t)pthread_cond_destroy(this_cond);
}
static inline sys_return_t condition_wait(condition_t *this_cond, mutex_t *this_mutex)
{
    return (sys_return_t)pthread_cond_wait(this_cond, this_mutex);
}
static inline sys_return_t condition_signal(condition_t *this_cond)
{
    return (sys_return_t)pthread_cond_signal(this_cond);
}
static inline sys_return_t condition_broadcast(condition_t *this_cond)
{
    return (sys_return_t)pthread_cond_broadcast(this_cond);
}

extern sys_return_t condition_timedwait(condition_t *cv, mutex_t *m, time_interval_t ns);

#ifdef __cplusplus
} // extern "C"
#endif

#endif // _OS_MUTEX_H_
