// Copyright 2013 Azul Systems, Inc.  All Rights Reserved.
#ifndef LICENSE_LICENSE_HPP
#define LICENSE_LICENSE_HPP 1

#include <zinglicense/license.h>
#include <aznix/az_memory.h>
#include <string>
#include <vector>
#include <inttypes.h>


typedef std::vector<unsigned char> ByteVector;

// Set to true to enable license debugging.  Default is off.
extern bool verboseLicense;

//-------------------------------------------------------------------------
// Methods for license consumers.
//-------------------------------------------------------------------------

bool findDefaultLicense (std::string &defaultLicenseFileName);
// Purpose:  Return the path to check for the default license;
// Params:   (Output) defaultLicenseFileName -- License file path.
// Returns:  true if successful, false otherwise.

bool programCanRunWithLicense (std::string licenseFileName,
                               zing_licensed_product_t programType,
                               bool skipInterfaceChecks,
                               bool quiet,
                               std::string &failureReason,
			       std::string &JARPath
			   );
// Purpose:  verify a license.
// Params:   (Input)  licenseFileName -- license to verify.
//                    programType     -- license type to check for.
//                    skipInterfaceChecks -- useful for validating a
//                                       license when it is created.  not
//                                       for use by a customer.
//                    quiet           -- don't print any warnings.
//           (Output) failureReason   -- if the validation failed, a description why.
// Returns:  true if the program should continue, 
//           false if the program should terminate.

bool isLicenseDirectory (const std::string licenseFileName);
// Pupose: Return true is licenseFileName is directory

void getLicenseFiles(std::vector<std::string> &licenseFiles, const std::string licenseFileName);         
// Purpose: Return the files in vector 'licenseFiles' from directory 'licenseFileName'

//-------------------------------------------------------------------------
// Methods for license producers.
//-------------------------------------------------------------------------

bool programCanRunWithLicense_internal (std::string licenseFileName,
                               zing_licensed_product_t programType,
                               bool skipInterfaceChecks,
                               bool quiet,
                               std::string &failureReason,
			       std::string &JARPath
			   );
// Purpose: Used within license library

ByteVector canonicalizeLicense (const std::string license);
// Purpose:  Do the ascii armoring.  Signing and verification should be done
//           on the armored byte vector.
// Params:   license -- raw license file from disk.
// Returns:  An armored ascii byte vector.

//-------------------------------------------------------------------------
// Generic helper methods.
//-------------------------------------------------------------------------

void printOpenSSLErrors();
// Purpose:  Print error diagnostic information.


std::string base64_decode(std::string const& encoded_string);
bool azlicense_canRunClass(const unsigned char *data, size_t size);
bool getLocalAmazonID(char *buf);
bool getLocalSystemUUID(char *buf);
bool exportCurlFuncs(void);
bool exportSSLFuncs(void);



//-------------------------------------------------------------------------
// Methods for license Utility.
//-------------------------------------------------------------------------
bool loadLicense (const std::string licenseFileName,
                  std::string &productName,
                  bool &trusted_customer,
                  time_t &warningTime,
                  time_t &expirationTime,
                  std::string &expirationDateString,
                  time_t &hardstopTime,
                  time_t &renewTime,
                  std::vector<std::string> &licensedMac,
                  std::vector<std::string> &licensedIpv4,
	                std::vector<std::string> &licensedIpv6,
	                std::vector<std::string> &licensedAmazonUUID,
									std::vector<std::string> &licensedSystemUUID,
									std::vector<std::string> &licensedAppSig,
	                std::string &JARPath,
	                std::string &failureReason,
	                std::string &licenseString
		);

#endif
