/* -*- mode: C; c-basic-offset:4; fill-column:78; -*- */
/** 
 * Copyright (c) 2004, 2005 Azul Systems, Inc.
 * All rights reserved.
 *
 * Azul Portable Runtime
 *
 * Author(s): bolsinga@azulsystems.com
 *
 * @file  psinfo.h (originally a part of azpr)
 * @brief AZPR get process information
 */

#include <stddef.h>
#include <inttypes.h>

#define HAS_PROC_FS 1

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*
 * Stats collection API.
 */

#define MAX_INTERFACE_STATS 64

typedef struct {
    char name[32];
    /* These values are for the system lifetime. */
    int64_t rx_bytes;                   /* ingress, into this system */
    int64_t rx_packets;
    int64_t tx_bytes;                   /* egress, out of this system */
    int64_t tx_packets;
} interface_stats_t;

typedef struct {
    /* cpu */
    size_t num_cpus;
    int64_t ticks_per_second;
    /* Ticks accumulated for the system, over all processes (and
       non-processes like idle).  Sum up the following four fields
       to get a total_ticks value.  
       
       So, percent_user over the system lifetime would be:
           (double)user_ticks / (double)total_ticks

       and percent_user for a window of time can be calculated as:
           (double)delta_user_ticks / (double)delta_total_ticks
    */
    int64_t user_ticks;                 /* user cpu time */
    int64_t other_ticks;                /* often i/o */
    int64_t system_ticks;               /* system cpu time */
    int64_t idle_ticks;

    /* load average.
       The 1, 5, and 15-minute load averages for the system. */
    double load_average_1_minute;
    double load_average_5_minute;
    double load_average_15_minute;

    /* memory */
    int64_t total_memory_bytes;
    int64_t used_memory_bytes;

    /* io */
    size_t num_interfaces;
    interface_stats_t interfaces[MAX_INTERFACE_STATS];
} system_stats_t;

typedef struct {
    /* cpu */
    int64_t user_ticks;                 /* sum for all threads */
    int64_t system_ticks;               /* sum for all threads */

    /* memory */
    int64_t virtual_memory_bytes;
    int64_t resident_memory_bytes;      /* aka physical memory */

    /* file descriptors */
    uint32_t open_file_descriptors;
} process_stats_t;

typedef struct {
    int64_t relative_time_nanos;        /* not necessarily comparable 
                                           with gettimeofday.  may be
                                           susceptible to clock drift. */
    system_stats_t system;
    process_stats_t process;
} system_process_stats_t;

int get_system_process_stats (system_process_stats_t *stats);

#ifdef __cplusplus
}
#endif /* __cplusplus */

