package Azul::PmemEntry;

use constant TYPE_BYTES => 1;
use constant TYPE_PERCENTAGE => 2;

my $MemTotal = 0;

package Azul::PmemReservedEntry;
use Exporter;

use os;
use strict;

# Variable
sub V {
    my $x = shift;
    die if !defined $x;
    die if defined shift;

    return $x . " ";
}

# Fixed
sub F {
    my $x = shift;
    die if !defined $x;
    die if defined shift;

    return $x . " ";
}

sub new {
    my $class = shift;
    die if !defined $class;
    die if defined shift;

    my $this = {
               };
    bless $this, $class;
    return $this;
}

sub setTypeBytes {
    my $this = shift;
    my $productName = shift;
    my $bytes = shift;
    die if !defined $bytes;
    die if defined shift;

    $this->setIfConfigured (false);
    $this->setProductName ($productName);
    $this->setType (Azul::PmemEntry::TYPE_BYTES);
    $this->setBytes ($bytes);

    my $MemTotal = $this->getMemTotal();
    my $percentage = int (($bytes / $MemTotal) * 100);
    $this->setPercentage ($percentage);
}

sub setTypePercentage {
    my $this = shift;
    my $productName = shift;
    my $percentage = shift;
    die if !defined $percentage;
    die if defined shift;

    $this->setIfConfigured (false);
    $this->setProductName ($productName);
    $this->setType (Azul::PmemEntry::TYPE_PERCENTAGE);
    $this->setPercentage ($percentage);

    my $MemTotal = $this->getMemTotal();
    my $bytes = int (($MemTotal * $percentage) / 100);
    $this->setBytes ($bytes);
}

sub setTypeIfConfigured {
    my $this = shift;
    my $productName = shift;
    die if !defined $productName;
    die if defined shift;

    $this->setIfConfigured (true);
    $this->setConfigured (false);
    $this->setProductName ($productName);
}

sub setTypeConfigured {
    my $this = shift;
    my $bytes = shift;
    die if !defined $bytes;
    die if defined shift;

    $this->setConfigured (true);
    $this->setType (Azul::PmemEntry::TYPE_BYTES);
    $this->setBytes ($bytes);

    my $MemTotal = $this->getMemTotal();
    my $percentage = int (($bytes / $MemTotal) * 100);
    $this->setPercentage ($percentage);
}

sub emit {
    my $this = shift;
    my $fh = shift;
    die if !defined $fh;
    die if defined shift;

    my $ifConfigured = $this->getIfConfigured();
    my $productName = $this->getProductName();

    if ($ifConfigured) {
        print $fh "reserved_if_configured ${productName}\n";
    }
    else {
        my $type = $this->getType();
        if ($type == Azul::PmemEntry::TYPE_BYTES) {
            my $bytes = $this->getBytes();
            print $fh "reserved ${productName} ${bytes}\n";
        }
        elsif ($type == Azul::PmemEntry::TYPE_PERCENTAGE) {
            my $percentage = $this->getPercentage();
            print $fh "reserved ${productName} ${percentage}\%   # of MemTotal\n";
        }
        else {
            die;
        }
    }
}

sub printTabular {
    my $this = shift;
    my $fh = shift;
    die if !defined $fh;
    die if defined shift;

    my $ifConfigured = $this->getIfConfigured();
    my $productName = $this->getProductName();

    if ($ifConfigured) {
        my $configured = $this->getConfigured();
        if ($configured) {
            my $bytes = $this->getBytes();
            my $percentage = $this->getPercentage();
            printf $fh ("%-20s %-20s %20s %6s\n",
                        "reserved (constant)", $productName, F($bytes), V($percentage . "\%"));
        }
        else {
            printf $fh ("%-20s %-20s %20s %6s\n",
                        "not configured", $productName, F(0), F(0 . "\%"));
        }
    }
    else {
        my $type = $this->getType();
        my $bytes = $this->getBytes();
        my $percentage = $this->getPercentage();
        if ($type == Azul::PmemEntry::TYPE_BYTES) {
            printf $fh ("%-20s %-20s %20s %6s\n",
                        "reserved", $productName, F($bytes), V($percentage . "\%"));
        }
        elsif ($type == Azul::PmemEntry::TYPE_PERCENTAGE) {
            printf $fh ("%-20s %-20s %20s %6s\n",
                        "reserved", $productName, V($bytes), F($percentage . "\%"));
        }
        else {
            die;
        }
    }
}

sub setIfConfigured {
    my $this = shift;
    my $value = shift;
    die if !defined $value;
    die if defined shift;

    $this->{"ifConfigured"} = $value;
}

sub getIfConfigured {
    my $this = shift;
    die if !defined $this;
    die if defined shift;

    return $this->{"ifConfigured"};
}

sub setConfigured {
    my $this = shift;
    my $value = shift;
    die if !defined $value;
    die if defined shift;

    $this->{"configured"} = $value;
}

sub getConfigured {
    my $this = shift;
    die if !defined $this;
    die if defined shift;

    return $this->{"configured"};
}

sub setProductName {
    my $this = shift;
    my $value = shift;
    die if !defined $value;
    die if defined shift;

    $this->{"productName"} = $value;
}

sub getProductName {
    my $this = shift;
    die if !defined $this;
    die if defined shift;

    return $this->{"productName"};
}

sub setType {
    my $this = shift;
    my $value = shift;
    die if !defined $value;
    die if defined shift;

    $this->{"type"} = $value;
}

sub getType {
    my $this = shift;
    die if !defined $this;
    die if defined shift;

    return $this->{"type"};
}

sub setBytes {
    my $this = shift;
    my $value = shift;
    die if !defined $value;
    die if defined shift;

    $this->{"bytes"} = $value;
}

sub getBytes {
    my $this = shift;
    die if !defined $this;
    die if defined shift;

    return $this->{"bytes"};
}

sub setPercentage {
    my $this = shift;
    my $value = shift;
    die if !defined $value;
    die if defined shift;

    $this->{"percentage"} = $value;
}

sub getPercentage {
    my $this = shift;
    die if !defined $this;
    die if defined shift;

    return $this->{"percentage"};
}

sub getMemTotal {
    return $Azul::PmemEntry::MemTotal;
}


package Azul::PmemFundEntry;

use os;
use strict;

sub V {
    my $x = shift;
    die if !defined $x;
    die if defined shift;

    return Azul::PmemReservedEntry::V ($x);
}

sub F {
    my $x = shift;
    die if !defined $x;
    die if defined shift;

    return Azul::PmemReservedEntry::F ($x);
}

sub new {
    my $class = shift;
    die if !defined $class;
    die if defined shift;

    my $this = {
               };
    bless $this, $class;
    return $this;
}

sub setTypeBytes {
    my $this = shift;
    my $fundName = shift;
    my $bytes = shift;
    die if !defined $bytes;
    die if defined shift;

    $this->setFundName ($fundName);
    $this->setType (Azul::PmemEntry::TYPE_BYTES);
    $this->setBytes ($bytes);
}

sub setTypePercentage {
    my $this = shift;
    my $fundName = shift;
    my $percentage = shift;
    die if !defined $percentage;
    die if defined shift;

    $this->setFundName ($fundName);
    $this->setType (Azul::PmemEntry::TYPE_PERCENTAGE);
    $this->setPercentage ($percentage);
}

sub emit {
    my $this = shift;
    my $fh = shift;
    die if !defined $fh;
    die if defined shift;

    my $fundName = $this->getFundName();
    my $type = $this->getType();
    if ($type == Azul::PmemEntry::TYPE_BYTES) {
        my $bytes = $this->getBytes();
        print $fh "fund ${fundName} ${bytes}\n";
    }
    elsif ($type == Azul::PmemEntry::TYPE_PERCENTAGE) {
        my $percentage = $this->getPercentage();
        print $fh "fund ${fundName} ${percentage}\%\n";
    }
    else {
        die;
    }
}

sub printTabular {
    my $this = shift;
    my $fh = shift;
    die if !defined $fh;
    die if defined shift;

    my $fundName = $this->getFundName();
    my $type = $this->getType();
    my $bytes = $this->getBytes();
    my $percentage = $this->getPercentage();
    if ($type == Azul::PmemEntry::TYPE_BYTES) {
        printf $fh ("%-20s %-20s %20s %6s\n",
                    "fund", $fundName, F($bytes), V($percentage . "\%"));
    }
    elsif ($type == Azul::PmemEntry::TYPE_PERCENTAGE) {
        printf $fh ("%-20s %-20s %20s %6s\n",
                    "fund", $fundName, V($bytes), F($percentage . "\%"));
    }
    else {
        die;
    }
}

sub setFundName {
    my $this = shift;
    my $value = shift;
    die if !defined $value;
    die if defined shift;

    $this->{"fundName"} = $value;
}

sub getFundName {
    my $this = shift;
    die if !defined $this;
    die if defined shift;

    return $this->{"fundName"};
}

sub setType {
    my $this = shift;
    my $value = shift;
    die if !defined $value;
    die if defined shift;

    $this->{"type"} = $value;
}

sub getType {
    my $this = shift;
    die if !defined $this;
    die if defined shift;

    return $this->{"type"};
}

sub setBytes {
    my $this = shift;
    my $value = shift;
    die if !defined $value;
    die if defined shift;

    $this->{"bytes"} = $value;
}

sub getBytes {
    my $this = shift;
    die if !defined $this;
    die if defined shift;

    return $this->{"bytes"};
}

sub setPercentage {
    my $this = shift;
    my $value = shift;
    die if !defined $value;
    die if defined shift;

    $this->{"percentage"} = $value;
}

sub getPercentage {
    my $this = shift;
    die if !defined $this;
    die if defined shift;

    return $this->{"percentage"};
}



package Azul::PmemConf;
# Changes for EFS
use File::Basename;

my $relocation_dir = dirname(__FILE__);

if ($relocation_dir ne "/usr/sbin/../lib/zing/perl5/Azul") {
        $relocation_dir =~ s!/usr/sbin/../lib/zing/perl5/Azul!!;
} else {
        $relocation_dir = "";
}

use os;
use Carp;
use strict;

my $AZPMEM = $relocation_dir . "/usr/sbin/zing-llpmem";

my $KB;
my $MB;
my $GB;
BEGIN {
    $KB = 1024;
    $MB = $KB * 1024;
    $GB = $MB * 1024;
}

use constant AZMM_PAGE_SIZE => 2 * $MB;
my $DEFAULT_NODEMASK = "FFFFFFFF";

sub V {
    my $x = shift;
    confess if !defined $x;
    die if defined shift;

    return Azul::PmemReservedEntry::V ($x);
}

sub new {
    my $class = shift;
    die if !defined $class;
    die if defined shift;

    my $this = {
                "enabled" => false,

                "memTotal" => undef,
                "memFree" => undef,

                "reserved" => [],
                "funds" => [],

                "memoryForLinuxBytes" => undef,
                "azulPmemPages" => undef,
                "fundMemoryType" => undef,       # Set when reading config file.
                "fundMemoryBytes" => undef,      # Set when reading config file.
                "fundMemoryPercentage" => undef, # Set when reading config file.
                "memoryForAzulBytes" => undef,
                "committedFundBytes" => undef,
                "committedFundPercentage" => undef,
                "nodemask" => $DEFAULT_NODEMASK, # Set when reading config file.
                "isElastic" => false,
               };

    bless $this, $class;

    return $this;
}

sub read {
    my $this = shift;
    die if !defined $this;
    my $fileName = shift;
    die if defined shift;

    if (!defined $fileName) {
        if (-f $_[0]) {
            $fileName = $_[0];
        }
    }

    if (!defined $fileName) {
        return false;
    }
    
    $this->resetFunds();
    $this->readMemInfo();
    $this->readConfig ($fileName);

    return true;
}

sub calculate {
    my $this = shift;
    my $errorStringRef = shift;
    die if !defined $errorStringRef;
    die if defined shift;

    # Calculate memory for Linux.
    my $MemTotal = $this->getMemTotal();
    my $memoryForLinuxBytes = 0;
    my $reserved = $this->getReserved();
    foreach my $entry (@$reserved) {
        my $productName = $entry->getProductName();

        if ($entry->getIfConfigured()) {
            if (! $entry->getConfigured()) {
                next;
            }
        }

        my $bytes = $entry->getBytes();
        $memoryForLinuxBytes += $bytes;

        if ($memoryForLinuxBytes >= $MemTotal) {
            $$errorStringRef = "Reserved linux memory exceeds MemTotal (product $productName)";
            return false;
        }
    }
    $this->setMemoryForLinuxBytes ($memoryForLinuxBytes);

    # Calculate accounted memory for Azul.
    my $desiredMemoryForAzulBytes;
    my $fundMemoryBytes = $this->getFundMemoryBytes();
    if (defined $fundMemoryBytes) {
        $desiredMemoryForAzulBytes = $fundMemoryBytes;
    }
    else {
        $desiredMemoryForAzulBytes = $MemTotal - $memoryForLinuxBytes;
    }
    my $azulPmemPages = int($desiredMemoryForAzulBytes / Azul::PmemConf::AZMM_PAGE_SIZE);
    $this->setAzulPmemPages ($azulPmemPages);
    my $memoryForAzulBytes = $azulPmemPages * Azul::PmemConf::AZMM_PAGE_SIZE;
    $this->setMemoryForAzulBytes ($memoryForAzulBytes);
    die if ($memoryForAzulBytes > $MemTotal);
    die if ($memoryForAzulBytes < 0);

    # Calculate fund by fund.
    my $remainingBytes = $memoryForAzulBytes;
    my $funds = $this->getFunds();
    foreach my $entry (@$funds) {
        my $fundName = $entry->getFundName();
        my $type = $entry->getType();

        my $bytes;
        if ($type == Azul::PmemEntry::TYPE_BYTES) {
            $bytes = $entry->getBytes();
            my $percentage = int (($bytes / $memoryForAzulBytes) * 100);
            $entry->setPercentage ($percentage);
        }
        elsif ($type == Azul::PmemEntry::TYPE_PERCENTAGE) {
            my $percentage = $entry->getPercentage();
            my $pages = int (($azulPmemPages * $percentage) / 100);
            $bytes = $pages * Azul::PmemConf::AZMM_PAGE_SIZE;
            $entry->setBytes ($bytes);
        }
        else {
            die;
        }

        $remainingBytes -= $bytes;

        if ($remainingBytes <= 0) {
            $this->setCommittedFundBytes (0);
            $this->setCommittedFundPercentage (0);
            $$errorStringRef = "Funds exceed total fund memory (fund $fundName)";
            return false;
        }
    }

    $this->setCommittedFundBytes ($remainingBytes);
    my $remainingPercentage = int (($remainingBytes / $memoryForAzulBytes) * 100);
    $this->setCommittedFundPercentage ($remainingPercentage);

    return true;
}

sub doit {
    my $this = shift;
    die if !defined $this;
    my $partition = shift;
    die if defined shift;

    if (!defined $partition) {
        if (-f $_[0]) {
            $partition = $_[0];
        }
    }
	
    if (!defined $partition) {
        return false;
    }

    # Skip if not enabled.
    my $enabled = $this->getEnabled();
    if (! $enabled) {
        os::info "Pmem is disabled.  Skipping.";
        return;
    }

    os::unsafeSystem($AZPMEM . " --partition $partition > /dev/null");
    if ($?) {
	os::fatal "Error Selecting Partition $partition";
	return;
    }

    # Skip if already initialized.
    os::unsafeSystem ($AZPMEM .  " --initialized --partition $partition > /dev/null");
    if (! $?) {
        os::fatal "Pmem is already initialized.  Skipping.";
	return;
    }

    my $elastic = $this->getIsElastic();
    if (! $elastic) {
		# Reserve pmem.
	    my $azulPmemPages = $this->getAzulPmemPages();
	    my $nodemask = $this->getNodemask();
	    os::info ("Zing Memory reserved using reserve-at-config policy");
	    os::info ("azulPmemPages: $azulPmemPages");
	    os::safeSystem ($AZPMEM,
			    "--reserve",
			    "--partition", $partition,
			    "--pages", $azulPmemPages,
			    "--nodemask", $nodemask);

	    my $initialized = false;
	    for (my $i = 0; $i < 10; $i++) {
		    os::unsafeSystem ($AZPMEM . " --initialized --partition $partition > /dev/null");
		    if (! $?) {
			    $initialized = true;
			    last;
		    }

		    os::info "Waiting for initialization, sleeping...";
		    sleep (1);
	    }

	    if (! $initialized) {
		    os::fatal "Failed to initialize azpmem";
	    }

	    $this->readMemInfo();

	# Move all fund bytes to a known bogus fund for starters.  This way if
	# something in the initialization fails, stuff won't accidentally run.
	    my $bogusFund = "7";
	    my $memoryForAzulBytes = $this->getMemoryForAzulBytes();
	    os::safeSystem ($AZPMEM,
			    "--transfer",
			    "--partition", $partition,
			    "--from", "Committed",
			    "--to", $bogusFund,
			    "--bytes", $memoryForAzulBytes);

		# Transfer fund by fund.
	    my $remainingBytes = $this->getMemoryForAzulBytes();
	    my $funds = $this->getFunds();
	    foreach my $entry (@$funds) {
		    my $fundName = $entry->getFundName();
		    my $bytes = $entry->getBytes();

		    os::unsafeSystem ($AZPMEM,
				    "--transfer",
				    "--partition", $partition,
				    "--from", $bogusFund,
				    "--to", $fundName,
				    "--bytes", $bytes);
		    if ($?) {
			    os::fatal ("Transfer to ${fundName} fund failed");
		    }

		    $remainingBytes -= $bytes;
	    }

	    my $committedFundBytes = $this->getCommittedFundBytes();
	    die if ($remainingBytes != $committedFundBytes);

		# Transfer everything left back to committed.
	    os::unsafeSystem ($AZPMEM,
			    "--transfer",
			    "--partition", $partition,
			    "--from", $bogusFund,
			    "--to", "Committed",
			    "--bytes", $committedFundBytes);
    } # ! $elastic
    else {
	    my $azulPmemPages = $this->getAzulPmemPages();
	    my $limitBytes = $azulPmemPages * Azul::PmemConf::AZMM_PAGE_SIZE;
	    my $nodemask = $this->getNodemask();
	    os::unsafeSystem($AZPMEM,
		    		 "--partition", $partition,
		   		 "--set-elastic", $nodemask);
	    if ($?) {
		    os::fatal ("Set Elastic failed");
	    }
	    os::unsafeSystem($AZPMEM,
		    		"--partition", $partition,
		    		"--set-device-limit", $limitBytes);
	    if ($?) {
		    os::fatal ("Set Device limit failed");
	    }

	    my $bogusFund = "7";
	    my $memoryForAzulBytes = $this->getMemoryForAzulBytes();
	    os::unsafeSystem ($AZPMEM,
				"--limit-transfer",
				"--partition", $partition,
				"--from", "Committed",
				"--to", $bogusFund,
				"--bytes", $memoryForAzulBytes);
	    if ($?) {
		    os::fatal ("Limit Transfer to $bogusFund fund failed");
	    }

	    # Transfer fund by fund.
	    my $remainingBytes = $this->getMemoryForAzulBytes();
	    my $funds = $this->getFunds();
	    foreach my $entry (@$funds) {
		    my $fundName = $entry->getFundName();
		    my $bytes = $entry->getBytes();

		    os::unsafeSystem ($AZPMEM,
				    "--limit-transfer",
				    "--partition", $partition,
				    "--from", $bogusFund,
				    "--to", $fundName,
				    "--bytes", $bytes);
		    if ($?) {
			os::fatal ("Limit Transfer to ${fundName} fund failed");
		    }

		    $remainingBytes -= $bytes;
	    }

	    my $committedFundBytes = $this->getCommittedFundBytes();
	    die if ($remainingBytes != $committedFundBytes);

	    # Transfer everything left back to committed.
	    os::unsafeSystem ($AZPMEM,
			    "--limit-transfer",
			    "--partition", $partition,
			    "--from", $bogusFund,
			    "--to", "Committed",
			    "--bytes", $committedFundBytes);

    } # $elastic
}

sub readMemInfo {
    my $this = shift;
    die if !defined $this;
    die if defined shift;

    my $MemTotal = 0;
    my $MemFree = 0;

    my $fh = new FileHandle ("/proc/meminfo");
    die if !defined $fh;
    while (<$fh>) {
        chomp $_;
        if (/MemTotal:\s*(\d+) kB/) {
            $MemTotal = $1 * $KB;
        }
        if (/MemFree:\s*(\d+) kB/) {
            $MemFree = $1 * $KB;
        }
    }
    close $fh;

    if (! $MemTotal) {
        os::fatal "Can't discover MemTotal";
    }

    if (! $MemFree) {
        os::fatal "Can't discover MemFree";
    }

    $Azul::PmemEntry::MemTotal = $MemTotal;
    $this->setMemTotal ($MemTotal);
    $this->setMemFree ($MemFree);
}

sub readConfig {
    my $this = shift;
    my $fileName = shift;
    die if !defined $fileName;
    die if defined shift;

    my $fd = new FileHandle ($fileName);
    if (! defined $fd) {
        os::fatal ("Cannot open $fileName");
    }

    my $enabled = false;

    # flag used to mark if fundmemory field in pmem.conf is elastic
    # or not
    my $is_fundmemory_elastic = false;
    # to keep tack of the number of funds (Grant and Pause Prevention)
    # Only one counter is required because, either funds will be in
    # percentage or in bytes. But not both in a valid pmem.conf file
    my $other_funds_count = 0;

    # Using flags below to determine if elastic or not based on the values
    my $fundmemory_is_elastic_as_per_values = undef;
    my $fundmemory_was_zero = undef;
    my $other_funds_are_elastic_as_per_values = undef;

    my $linenum = 0;
    while (<$fd>) {
        $linenum++;
        my $line = $_;
        chomp $line;            # Trim newline.
        $line =~ s/#.*//;       # Trim comments.
        $line =~ s/^\s*//;      # Trim leading whitespace.
        $line =~ s/\s*$//;      # Trim trailing whitespace.

        if (length ($line) == 0) {
            next;               # Skip blank lines.
        }

        if ($line =~ /^(pmem)\s+(\S+)$/) {
            my $action = $2;
            if ($action eq "enabled") {
                $this->setEnabled (true);
            }
            elsif ($action eq "disabled") {
                $this->setEnabled (false);
            }
            else {
                os::fatal "Parse error (pmem directive) on file ${fileName} line ${linenum}";
            }
        }
        elsif ($line =~ /^(reserved)\s+(\S+)\s+(\d+[mMgG]?)$/) {
            my $product = $2;
            my $expr = $3;
            my $bytes;
            if ($expr =~ /(\d+)[mM]/) {
                $bytes = $1 * $MB;
            }
            elsif ($expr =~ /(\d+)[gG]/) {
                $bytes = $1 * $GB;
            }
            elsif ($expr =~ /(\d+)/) {
                $bytes = $1;
            }
            else {
                die;
            }

            my $entry = new Azul::PmemReservedEntry;
            $entry->setTypeBytes ($product, $bytes);
            $this->addReserved ($entry);
        }
        elsif ($line =~ /^(reserved)\s+(\S+)\s+(\d+)%$/) {
            my $product = $2;
            my $percentage = $3;
            if (($percentage < 0) || ($percentage > 100)) {
                os::fatal "Parse error (bad percentage) on file ${fileName} line ${linenum}";
            }
            my $entry = new Azul::PmemReservedEntry;
            $entry->setTypePercentage ($product, $percentage);
            $this->addReserved ($entry);
        }
        elsif ($line =~ /^(reserved_if_configured)\s+(\S+)$/) {
            my $product = $2;

            my $entry = new Azul::PmemReservedEntry;
            $entry->setTypeIfConfigured ($product);
            $this->addReserved ($entry);

            my $initFile = "/etc/init.d/${product}";
            if (! -f $initFile) {
                next;
            }

            os::unsafeSystem ("/etc/init.d/${product} is-pmem-needed 1> /dev/null 2> /dev/null");
            if ($? == 0) {
                my $bytes = `/etc/init.d/${product} pmem`;
                if ($?) {
                    os::error "Could not determine pmem required for ${product}";
                    next;
                }
                chomp $bytes;

                $entry->setTypeConfigured ($bytes);
            }
        }
        elsif ($line =~ /^(fundmemory)\s+(\S+)$/ || $line =~ /^(fundmemory)\s+(\S+)\s+(\S+)$/) {
            my $expr = undef;
            if ($line =~ /^(fundmemory)\s+(\S+)$/) {
                $expr = $2;
            }
            elsif ($line =~ /^(fundmemory)\s+(\S+)\s+(\S+)$/) {
                $expr = $3;
                my $expr_low_limit = $2;

                my $value = undef;
                my $value_low_limit = undef;

                $is_fundmemory_elastic = true;

                if ($expr =~ /^(\d+)/) {
                    $value = $1;
                }
                if ($expr_low_limit =~ /^(\d+)/) {
                    $value_low_limit = $1;
                }
                if ($value == $value_low_limit) {
                    if ($value == 0 && $value_low_limit == 0) {
                        $fundmemory_was_zero = true;
                    } else {
                        $fundmemory_was_zero = false;
                    }
                    $fundmemory_is_elastic_as_per_values = false;
                } elsif ($value_low_limit == 0) {
                    $fundmemory_was_zero = true;
                    $fundmemory_is_elastic_as_per_values = true;
                } else {
                    os::fatal "Parse error (Invalid value $value_low_limit, should be either 0 or the upper limit $value) on file ${fileName} line ${linenum}";
                }
            }
            else {
                os::fatal "Parse error (unexpected pattern) on file ${fileName} line ${linenum}";
            }

            my $MemTotal = $this->getMemTotal();
            my $desiredMemoryForAzulBytes;
            if ($expr =~ /^(\d+)%$/) {
                my $percentage = $1;
                if (($percentage < 0) || ($percentage > 100)) {
                    os::fatal "Parse error (bad percentage) on file ${fileName} line ${linenum}";
                }

                $this->setFundMemoryType (Azul::PmemEntry::TYPE_PERCENTAGE);
                $this->setFundMemoryPercentage ($percentage);

                # Calculate accounted memory for Azul.
                $desiredMemoryForAzulBytes = ($MemTotal * $percentage / 100);
            }
            elsif ($expr =~ /^(\d+[mMgG]?)$/) {
                my $expr2 = $1;

                if ($expr2 =~ /(\d+)[mM]/) {
                    $desiredMemoryForAzulBytes = $1 * $MB;
                }
                elsif ($expr2 =~ /(\d+)[gG]/) {
                    $desiredMemoryForAzulBytes = $1 * $GB;
                }
                elsif ($expr2 =~ /(\d+)/) {
                    $desiredMemoryForAzulBytes = $1;
                }
                else {
                    die;
                }

                if (($desiredMemoryForAzulBytes / Azul::PmemConf::AZMM_PAGE_SIZE) !=
                    int ($desiredMemoryForAzulBytes / Azul::PmemConf::AZMM_PAGE_SIZE)) {
                    os::fatal "Parse error (value not a multiple of 2MB) on file ${fileName} line ${linenum}";
                }

                $this->setFundMemoryType (Azul::PmemEntry::TYPE_BYTES);
            }

            my $azulPmemPages = int($desiredMemoryForAzulBytes / Azul::PmemConf::AZMM_PAGE_SIZE);
            my $memoryForAzulBytes = $azulPmemPages * Azul::PmemConf::AZMM_PAGE_SIZE;
            $this->setFundMemoryBytes ($memoryForAzulBytes);
        }
        elsif ($line =~ /^(fund)\s+(\S+)\s+(\d+)%$/ || $line =~ /^(fund)\s+(\S+)\s+(\d+)%\s+(\d+)%$/) {
            my $fundName = undef;
            my $percentage = undef;
            if ($line =~ /^(fund)\s+(\S+)\s+(\d+)%$/) {
                $fundName = $2;
                $percentage = $3;
            }
            elsif ($line =~ /^(fund)\s+(\S+)\s+(\d+)%\s+(\d+)%$/) {
                $fundName = $2;
                $percentage = $4;
                my $percentage_low_limit = $3;

                my $value = undef;
                my $value_low_limit = undef;

                if ($percentage =~ /^(\d+)/) {
                    $value = $1;
                }
                if ($percentage_low_limit =~ /^(\d+)/) {
                    $value_low_limit = $1;
                }

                if ($value != 0) {
                  if ($fundmemory_was_zero == false && $value_low_limit == 0) {
                    os::fatal "Parse error ($fundName should have non-zero value) on file ${fileName} line ${linenum}";
                  } elsif ($fundmemory_was_zero == true && $value_low_limit !=0) {
                    os::fatal "Parse error ($fundName should have zero value) on file ${fileName} line ${linenum}";
                  }
                }

                if ($value_low_limit == $value) {
                    # both values set to '0' are valid
                    if ($value_low_limit == 0 && $value == 0) {
                        $other_funds_are_elastic_as_per_values = true;
                    } else {
                        $other_funds_are_elastic_as_per_values = false;
                    }
                } elsif ($value_low_limit == 0) {
                    # if flag other_funds_are_elastic_as_per_values is already set to false by the
                    # previous iteration then don't change it to true. 
                    if ($other_funds_count == 1 && $other_funds_are_elastic_as_per_values == false) {
                        $other_funds_are_elastic_as_per_values = false;
                    } else {
                        $other_funds_are_elastic_as_per_values = true;
                    }
                } else {
                    os::fatal "Parse error (Invalid value $value_low_limit% of $fundName, it can only be 0% or the upper limit $value%) on file ${fileName} line ${linenum}";
                }

                $other_funds_count = $other_funds_count+1;
            }
            else {
                os::fatal "Parse error (unexpected pattern) on file ${fileName} line ${linenum}";
            }

            if (($fundName ne "Grant") &&
                ($fundName ne "PausePrevention")) {
                os::fatal "Parse error (unknown fund name) on file ${fileName} line ${linenum}";
            }

            if (($percentage < 0) || ($percentage > 100)) {
                os::fatal "Parse error (bad percentage) on file ${fileName} line ${linenum}";
            }

            my $entry = new Azul::PmemFundEntry;
            $entry->setTypePercentage ($fundName, $percentage);
            $this->addFund ($entry);
        }
        elsif ($line =~ /^(fund)\s+(\S+)\s+(\d+[mMgG]?)$/ || $line =~ /^(fund)\s+(\S+)\s+(\d+[mMgG]?)\s+(\d+[mMgG]?)$/) {
            my $fundName = undef;
            my $expr = undef;
            if ($line =~ /^(fund)\s+(\S+)\s+(\d+[mMgG]?)$/) {
                $fundName = $2;
                $expr = $3;
            }
            elsif ($line =~ /^(fund)\s+(\S+)\s+(\d+[mMgG]?)\s+(\d+[mMgG]?)$/) {
                $fundName = $2;
                $expr = $4;
                my $expr_low_limit = $3;

                my $value = undef;
                my $value_low_limit = undef;

                if ($expr =~ /^(\d+)/) {
                    $value = $1;
                }
                if ($expr_low_limit =~ /^(\d+)/) {
                    $value_low_limit = $1;
                }

                if ($value != 0) {
                    if ($fundmemory_was_zero == false && $value_low_limit == 0) {
                        os::fatal "Parse error ($fundName should have non-zero value) on file ${fileName} line ${linenum}";
                    } elsif ($fundmemory_was_zero == true && $value_low_limit !=0) {
                        os::fatal "Parse error ($fundName should zero value) on file ${fileName} line ${linenum}";
                    }
                }
                
                if ($value_low_limit == $value) {
                    # both values set to '0' are valid
                    if ($value_low_limit == 0 && $value == 0) {
                        $other_funds_are_elastic_as_per_values = true;
                    } else {
                        $other_funds_are_elastic_as_per_values = false;
                    }
                } elsif ($value_low_limit == 0) {
                    # if flag other_funds_are_elastic_as_per_values is already set to false by the
                    # previous iteration then don't change it to true. 
                    if ($other_funds_count == 1 && $other_funds_are_elastic_as_per_values == false) {
                        $other_funds_are_elastic_as_per_values = false;
                    } else {
                        $other_funds_are_elastic_as_per_values = true;
                    }
                } else {
                    os::fatal "Parse error (Invalid value $value_low_limit of $fundName, it can only be 0 or the upper limit $value) on file ${fileName} line ${linenum}";
                }
 
                $other_funds_count = $other_funds_count+1;
            }
            else {
                os::fatal "Parse error (unexpected pattern) on file ${fileName} line ${linenum}";
            }
            my $bytes;
            if ($expr =~ /(\d+)[mM]/) {
                $bytes = $1 * $MB;
            }
            elsif ($expr =~ /(\d+)[gG]/) {
                $bytes = $1 * $GB;
            }
            elsif ($expr =~ /(\d+)/) {
                $bytes = $1;
            }
            else {
                die;
            }

            if (($fundName ne "Grant") &&
                ($fundName ne "PausePrevention")) {
                os::fatal "Parse error (unknown fund name) on file ${fileName} line ${linenum}";
            }

            if ($bytes < 0) {
                os::fatal "Parse error (bad bytes) on file ${fileName} line ${linenum}";
            }

            my $entry = new Azul::PmemFundEntry;
            $entry->setTypeBytes ($fundName, $bytes);
            $this->addFund ($entry);
        }
	      elsif ($line =~ /^(nodemask)\s+(\S+)$/) {
            my $nodemask = $2;
            $this->setNodemask($nodemask);
    	  }
        else {
            os::fatal "Parse error on file ${fileName} line ${linenum}";
        }
    }
    
    # pmem.conf will have valid elastic parameters only when $is_fundmemory_elastic is true and
    # count of other funds (Grant and Pause Prevention) is equal to 2. Else, will be non-elastic
    # iff $is_fundmemory_elastic is false and count of other funds is not 2. Otherwise, it is an
    # invalid pmem.conf file.
    if (($is_fundmemory_elastic == true) && ($other_funds_count == 2)) { 
        if ($fundmemory_is_elastic_as_per_values == false || $other_funds_are_elastic_as_per_values == false) {
            $this->setIsElastic (false);
        } else {
            $this->setIsElastic (true);
        }
    }
    elsif (($is_fundmemory_elastic == false) && ($other_funds_count != 2)) { 
        # Explicitly setting the value to false. 
        $this->setIsElastic (false);
        # when only one elastic limit is read for Grant or Pause Prevention
        if ($other_funds_count != 0) {
            os::fatal "Parse error (invalid elastic limits) on file ${fileName} line ${linenum}";
        }
    }
    else {
            os::fatal "Parse error (invalid elastic limits) on file ${fileName} line ${linenum}";
    }

    close $fd;
}

sub emit {
    my $this = shift;
    my $fh = shift;
    die if !defined $fh;
    die if defined shift;

    my $enabled = $this->getEnabled();
    print $fh "pmem ", ($enabled ? "enabled" : "disabled"), "\n";
    print $fh "\n";

    my $reserved = $this->getReserved();
    foreach my $entry (@$reserved) {
        $entry->emit ($fh);
    }
    print $fh "\n";

    my $fundMemoryType = $this->getFundMemoryType();
    if (defined $fundMemoryType) {
        if ($fundMemoryType == Azul::PmemEntry::TYPE_BYTES) {
            my $fundMemoryBytes = $this->getFundMemoryBytes();
            print $fh "fundmemory ${fundMemoryBytes}\n";
        }
        else {
            my $fundMemoryPercentage = $this->getFundMemoryPercentage();
            print $fh "fundmemory ${fundMemoryPercentage}\%   # of MemTotal\n";
        }
    }

    my $funds = $this->getFunds();
    foreach my $entry (@$funds) {
        $entry->emit ($fh);
    }
    my $nodemask = $this->getNodemask();
    if ($nodemask eq $DEFAULT_NODEMASK) {
	    print $fh "\nnodemask	ALL\n";
    } else {
	    print $fh "\nnodemask	${nodemask}\n";
    }
}

sub gb {
    my $bytes = shift;
    die if !defined $bytes;
    die if defined shift;

    my $gigabytes = int ($bytes / (1024 * 1024 * 1024) * 10 + 0.5) / 10;
    return "${gigabytes} GB";
}

sub printTabular {
    my $this = shift;
    my $fh = shift;
    die if !defined $fh;
    die if defined shift;

    printf $fh ("%-20s %-20s %20s %6s\n",
                "STATUS", "PRODUCT", "BYTES ", "PCT_OF_MemTotal");
    print $fh "\n";
    my $reserved = $this->getReserved();
    foreach my $entry (@$reserved) {
        $entry->printTabular ($fh);
    }
    print $fh "\n\n";

    printf $fh ("%-20s %-20s %20s %6s\n",
                "STATUS", "FUND_NAME", "BYTES ", "PCT_OF_FundMem");
    print $fh "\n";
    my $funds = $this->getFunds();
    foreach my $entry (@$funds) {
        $entry->printTabular ($fh);
    }

    my $bytes = $this->getCommittedFundBytes();
    my $percentage = $this->getCommittedFundPercentage();
    printf $fh ("%-20s %-20s %20s %6s\n",
                "fund", "Committed", V($bytes), V($percentage . "\%"));

    print $fh "\n\n";
    my $MemTotal = $this->getMemTotal();
    printf $fh ("MemTotal: %15s (%s)\n", $MemTotal, gb($MemTotal));
    my $memoryForAzulBytes = $this->getMemoryForAzulBytes();
    printf $fh ("FundMem:  %15s (%s)\n", $memoryForAzulBytes, gb($memoryForAzulBytes));
};

sub findReservedByName {
    my $this = shift;
    my $name = shift;
    die if !defined $name;
    die if defined shift;

    my $reserved = $this->getReserved();
    foreach my $entry (@$reserved) {
        if ($entry->getProductName() eq $name) {
            return $entry;
        }
    }

    return undef;
}

sub findFundByName {
    my $this = shift;
    my $name = shift;
    die if !defined $name;
    die if defined shift;

    my $funds = $this->getFunds();
    foreach my $entry (@$funds) {
        if ($entry->getFundName() eq $name) {
            return $entry;
        }
    }

    return undef;
}

sub getMemTotal {
    my $this = shift;
    die if !defined $this;
    die if defined shift;

    return $this->{"memTotal"};
}

sub setMemTotal {
    my $this = shift;
    my $value = shift;
    die if !defined $value;
    die if defined shift;

    $this->{"memTotal"} = $value;
}

sub getMemFree {
    my $this = shift;
    die if !defined $this;
    die if defined shift;

    return $this->{"memFree"};
}

sub setMemFree {
    my $this = shift;
    my $value = shift;
    die if !defined $value;
    die if defined shift;

    $this->{"memFree"} = $value;
}

sub getEnabled {
    my $this = shift;
    die if !defined $this;
    die if defined shift;

    return $this->{"enabled"};
}

sub setEnabled {
    my $this = shift;
    my $value = shift;
    die if !defined $value;
    die if defined shift;

    $this->{"enabled"} = $value;
}

sub getPmemPages {
    my $this = shift;
    die if !defined $this;
    die if defined shift;

    return $this->{"pmemPages"};
}

sub setPmemPages {
    my $this = shift;
    my $value = shift;
    die if !defined $value;
    die if defined shift;

    $this->{"pmemPages"} = $value;
}

sub getReserved {
    my $this = shift;
    die if !defined $this;
    die if defined shift;

    return $this->{"reserved"};
}

sub addReserved {
    my $this = shift;
    my $entry = shift;
    die if !defined $entry;
    die if defined shift;

    my $reserved = $this->getReserved();
    push @$reserved, $entry;
}

sub resetFunds {
    my $this = shift;
    die if !defined $this;
    die if defined shift;

    $this->{"funds"} = [];
}

sub getFunds {
    my $this = shift;
    die if !defined $this;
    die if defined shift;

    return $this->{"funds"};
}

sub addFund {
    my $this = shift;
    my $entry = shift;
    die if !defined $entry;
    die if defined shift;

    my $funds = $this->getFunds();
    push @$funds, $entry;
}

sub getMemoryForLinuxBytes {
    my $this = shift;
    die if !defined $this;
    die if defined shift;

    return $this->{"memoryForLinuxBytes"};
}

sub setMemoryForLinuxBytes {
    my $this = shift;
    my $value = shift;
    die if !defined $value;
    die if defined shift;

    $this->{"memoryForLinuxBytes"} = $value;
}

sub getFundMemoryType {
    my $this = shift;
    die if !defined $this;
    die if defined shift;

    return $this->{"fundMemoryType"};
}

sub setFundMemoryType {
    my $this = shift;
    my $value = shift;
    die if !defined $value;
    die if defined shift;

    $this->{"fundMemoryType"} = $value;
}

sub getFundMemoryBytes {
    my $this = shift;
    die if !defined $this;
    die if defined shift;

    return $this->{"fundMemoryBytes"};
}

sub setFundMemoryBytes {
    my $this = shift;
    my $value = shift;
    die if !defined $value;
    die if defined shift;

    $this->{"fundMemoryBytes"} = $value;
}

sub getFundMemoryPercentage {
    my $this = shift;
    die if !defined $this;
    die if defined shift;

    return $this->{"fundMemoryPercentage"};
}

sub setFundMemoryPercentage {
    my $this = shift;
    my $value = shift;
    die if !defined $value;
    die if defined shift;

    $this->{"fundMemoryPercentage"} = $value;
}

sub getAzulPmemPages {
    my $this = shift;
    die if !defined $this;
    die if defined shift;

    return $this->{"azulPmemPages"};
}

sub setAzulPmemPages {
    my $this = shift;
    my $value = shift;
    die if !defined $value;
    die if defined shift;

    $this->{"azulPmemPages"} = $value;
}

sub getMemoryForAzulBytes {
    my $this = shift;
    die if !defined $this;
    die if defined shift;

    return $this->{"memoryForAzulBytes"};
}

sub setMemoryForAzulBytes {
    my $this = shift;
    my $value = shift;
    die if !defined $value;
    die if defined shift;

    $this->{"memoryForAzulBytes"} = $value;
}

sub getCommittedFundBytes {
    my $this = shift;
    die if !defined $this;
    die if defined shift;

    return $this->{"committedFundBytes"};
}

sub setCommittedFundBytes {
    my $this = shift;
    my $value = shift;
    die if !defined $value;
    die if defined shift;

    $this->{"committedFundBytes"} = $value;
}

sub getCommittedFundPercentage {
    my $this = shift;
    die if !defined $this;
    die if defined shift;

    return $this->{"committedFundPercentage"};
}

sub setCommittedFundPercentage {
    my $this = shift;
    my $value = shift;
    die if !defined $value;
    die if defined shift;

    $this->{"committedFundPercentage"} = $value;
}

sub setNodemask {
	my $this = shift;
	my $value = shift;
	die if !defined $value;
	die if defined shift;

	if ($value ne "ALL") {
		$this->{"nodemask"} = $value;
	} else {
		$this->{"nodemask"} = "0xFFFFFFFF";
	}
}

sub getNodemask {
	my $this = shift;
	die if !defined $this;
	die if defined shift;

	return $this->{"nodemask"};
}

sub getIsElastic {
    my $this = shift;
    die if !defined $this;
    die if defined shift;

    return $this->{"isElastic"};
}

sub setIsElastic {
    my $this = shift;
    my $value = shift;
    die if !defined $value;
    die if defined shift;

    $this->{"isElastic"} = $value;
}

1;
