package Azul::WizardHelpers;

use Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(query_border
             query_value_header
             query_value
             query_error_message
             query_yes_no
           );
use strict;
use English qw( -no_match_vars ); # Avoids regex performance penalty

sub trim {
    my $str = shift;
    die if !defined $str;

    $str =~ s/^\s+//;
    $str =~ s/\s+$//;
    return $str;
}

sub query_string {
    my $header = shift;
    my $prompt = shift;
    my $sub_ref = shift;
    my @args = @_;
    my $result;
    STRING: {
        if ($header) {
            print "\n** $header\n\n";
        }
        my $spacer = ' ' x (10 - length($prompt));
        print "=> $prompt$spacer : ";

        my $choice = <STDIN>;
        redo STRING if (!defined $choice);

        $choice = trim($choice);

        eval {
            $result = &{$sub_ref}($choice, @args);
        };
        if ($EVAL_ERROR) {
            print "!! $EVAL_ERROR";
            redo STRING;
        }
    }
    return $result;
}

sub query_choice {
    my $prompt = shift;
    my $regulars_ref = shift;
    my $specials_ref = shift;
    CHOICE: {
        print "\n** $prompt\n\n";
        my %choices = ();
        # regular options
        my $choice = 1;
        my @regulars = @{$regulars_ref};
        for (my $i = 0; $i < $#regulars; $i += 2) {
            print "\t[", $choice, "]\t", $regulars[$i], "\n";
            $choices{$choice} = $regulars[$i + 1];
            $choice++;
        }
        if (defined $specials_ref) {
            # special options
            $choice = 0;
            my @specials = @{$specials_ref};
            for (my $i = 0; $i < $#specials; $i += 2) {
                $choices{$choice} = $specials[$i + 1];
                $choice--;
                if ($choice < 0) {
                    $choice += 10;
                }
            }
            for (my $i = $#specials - 1; $i >= 0; $i -= 2) {
                $choice++;
                if ($choice >= 10) {
                    $choice -= 10;
                }
                print "\t[", $choice, "]\t", $specials[$i], "\n";
            }
        }
        print "\n=> Selection  : ";
        $choice = <STDIN>;
        redo CHOICE if (!defined $choice);

        $choice = trim($choice);
        if ($choice =~ /^\d$/) {
            my $result = $choices{$choice};
            return $result if (defined $result);
        }
        print "\n!! Unrecognized selection\n";
        redo CHOICE;
    }
}

sub query_border {
    my $task = shift;
    my $subtask = shift;
    die if defined shift;

    print "\n";
    print "---------------------------------------------------------------------------\n";
    print "\n";
    my $str = "";
    if (defined $task) {
        $str = $task;
    }
    if (defined $subtask) {
        $str .= " / ${subtask}";
    }
    if (length ($str) > 0) {
        print "[ Current task:  ${str} ]\n";
        print "\n";
    }
}

sub query_value_header {
    my $task = shift;
    my $subtask = shift;
    my $name = shift;
    my $description = shift;
    die if !defined $description;
    my $default = shift;
    die if defined shift;

    my $defaultToPrint;
    if (defined $default) {
        $defaultToPrint = $default;
    }
    else {
        $defaultToPrint = "<no default>";
    }

    query_border ($task, $subtask);
    print "Description:\n";
    print "${description}";
    print "\n";
    print "Default value:  ${defaultToPrint}\n";
    print "\n";
}

sub query_value_undefined_ok {
    my $name = shift;
    die if !defined $name;
    my $default = shift;
    die if defined shift;

    my $msg = "** Enter ${name}";
    if (defined $default) {
        $msg .= " [default '${default}']";
    }
    $msg .= ": ";
    print $msg;

    my $value = <STDIN>;

    if (defined $value) {
        $value = trim($value);
    }
    else {
        return undef;
    }

    if (length ($value) == 0) {
        if (defined $default) {
            $value = $default;
        }
    }

    return $value;
}

sub query_error_message {
    my $name = shift;
    my $message = shift;
    die if !defined $message;

    print ("!! Input error: ${message}\n");
}

sub query_value {
    my $name = shift;
    die if !defined $name;
    my $default = shift;
    die if defined shift;

    while (1) {
        my $value = query_value_undefined_ok ($name, $default);
        if (defined $value) {
            return $value;
        }

        query_error_message ($name, "Value must be defined");
    }
}

sub query_yes_no {
    my $question = shift;
    die if !defined $question;
    my $default = shift;
    die if defined shift;

    while (1) {
        print "** ${question}\n";
        my $msg = "** (y)es or (n)o";
        if (defined $default) {
            $msg .= " [default '${default}']";
        }
        $msg .= ": ";
        print $msg;

        my $value = <STDIN>;
        $value = trim($value);

        if (! length ($value)) {
            if (defined $default) {
                $value = $default;
            }
        }

        if (($value eq "y") || ($value eq "yes") || ($value eq "Y")) {
            return 1;
        }

        if (($value eq "n") || ($value eq "no") || ($value eq "N")) {
            return 0;
        }

        print "!! Error: You must answer yes or no\n";
    }
}

1;
